//--------------------------------------------------------------
// File     : stm32_ub_basic.c
// Datum    : 20.12.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : STM32_UB_BASIC_EDITOR,
// Funktion : Basic Editor/Interpreter
//
// uBasic   : Adam Dunkels (http://dunkels.com/adam/ubasic/)
//
// Linker-Script : "ub_linkerscript_stm32f429i_disco.ld"
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_basic.h"




//--------------------------------------------------------------
// Directorys und Files im FLASH
// (Files auch im H-File hinzufuegen)
//--------------------------------------------------------------
const struct BASIC_ROM_t BASIC_ROM[] = {
  //---------------------------------------------
  {0,0,1,"games", 0},             // Dir #1
  {0,1,0,"..", 0},                // Backdir
  {1,1,1,"breakout.bas", &breakout},
  {1,1,1,"hangman.bas", &hangman},
  {1,1,1,"paint.bas", &paint},
  {1,1,1,"pong.bas", &pong},
  //---------------------------------------------
  {0,0,2,"level0", 0},            // Dir #2
  {0,2,0,"..", 0},                // Backdir
  {1,2,2,"calc.bas", &calc},
  {1,2,2,"gpio.bas", &gpio},
  {1,2,2,"graphic.bas", &graphic},
  {1,2,2,"key.bas", &key},
  {1,2,2,"logic.bas", &logic},
  //---------------------------------------------
  {0,0,3,"level1", 0},            // Dir #3
  {0,3,0,"..", 0},                // Backdir
  {1,3,3,"clock.bas", &clock},
  {1,3,3,"i2c.bas", &i2c},
  {1,3,3,"string.bas", &string_bas},
  {1,3,3,"uart.bas", &uart},
  //---------------------------------------------
  {0,0,4,"level2", 0},            // Dir #4
  {0,4,0,"..", 0},                // Backdir
  {1,4,4,"sprite.bas", &sprite},
  //---------------------------------------------
  {1,0,0,"hello.bas", &hello}     // File in root
};
const uint32_t BASIC_ROM_ITEMS=sizeof(BASIC_ROM)/sizeof(BASIC_ROM[0]);


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void p_Basic_init_HW(void);
void p_Basic_init_SW(uint8_t mode);
void p_Basic_check_Keyboard(void);
void p_Basic_check_Keyboard_LIST_DIR_Mode(void);
void p_Basic_check_Keyboard_RUN_Mode(uint8_t mode);
void p_Basic_line_Interpreter(uint8_t mode);
void p_Basic_refresh_LCD(void);
void p_Basic_show_Header(void);
void p_Basic_show_Version(void);
void p_Basic_show_Copyright(void);
void p_Basic_show_Info(void);
void p_Basic_show_Ok(void);
void p_Basic_show_Ok2(void);
void p_Basic_show_Error(void);
void p_Basic_show_Fail(void);
void p_Basic_show_UARTLine(void);
void p_Basic_clear_BasicRAM(void);
void p_Basic_set_Terminal(void);
void p_Basic_show_Listing(uint8_t mode);
void p_Basic_show_Directory(uint8_t mode);
void p_Basic_change_Device(void);
void p_Basic_delete_File(void);
void p_Basic_change_Dir(void);
void p_Basic_load_Program(void);
void p_Basic_load_From_UART(uint8_t mode);
void p_Basic_load_From_FLASH(void);
void p_Basic_load_From_SD(void);
void p_Basic_check_BasicRAM(void);
void p_Basic_save_Program(void);
void p_Basic_save_To_UART(void);
void p_Basic_save_To_SD(void);
void p_Basic_parse_Program(void);
void p_Basic_run_Program(void);
void p_Basic_send_Screen(void);
uint8_t p_Basic_check_String(uint8_t mode);



//--------------------------------------------------------------
// Speicher fuer Basic-Programm (im externen SDRAM)
//--------------------------------------------------------------
char basic_ram[STM_BASIC_RAM_SIZE] __attribute__((section(".HeapMemSection")));
//--------------------------------------------------------------
// speicherplatz fuer eine einzelne Zeile
//--------------------------------------------------------------
char inline_interpr[100] __attribute__((section(".HeapMemSection")));




//--------------------------------------------------------------
// Header fuer BMP-Transfer
// (fix als einen kompletten Screen (320x240) im Landscape-Mode)
//--------------------------------------------------------------
const uint8_t BMP_HEADER[54]={
0x42,0x4D,0x36,0x84,0x03,0x00,0x00,0x00,0x00,0x00, // ID=BM, Filsize=(240x320x3+54)
0x36,0x00,0x00,0x00,0x28,0x00,0x00,0x00,           // Offset=54d, Headerlen=40d
0x40,0x01,0x00,0x00,0xF0,0x00,0x00,0x00,0x01,0x00, // W=320d, H=240d (landscape)
0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x84,0x03,0x00, // 24bpp, unkomprimiert, Data=(240x320x3)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,           // nc
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};          // nc


//--------------------------------------------------------------
char line_buf[85]; // Zwischenpuffer fuer Strings


//--------------------------------------------------------------
// init vom Basic
//--------------------------------------------------------------
void UB_Basic_Init(void)
{
  // init der Software
  p_Basic_init_SW(0);

  // init vom Editor
  UB_Basic_Editor_Init();

  // init der Hardware
  p_Basic_init_HW();

  // loeschen vom LCD-RAM
  UB_Basic_Editor_ClearSreen();

  // loeschen vom Basic-RAM
  p_Basic_clear_BasicRAM();

  // Header anzeigen
  p_Basic_show_Header();

  // Timer fuer Screen Refresh freigeben
  UB_BASIC.mode=BASIC_MODE_EDITOR;
}


//--------------------------------------------------------------
// start vom Basic (dauerloop)
//--------------------------------------------------------------
void UB_Basic_Start(void)
{
  while(1) {

    if(UB_BASIC.mode==BASIC_MODE_EDITOR) {
      //---------------------------------------------------
      // Editor-Mode
      //---------------------------------------------------      


      //---------------------------------------------------
      // Tastatur bearbeiten
      //---------------------------------------------------
      if(akt_usb_status==USB_HID_KEYBOARD_CONNECTED) {
        UB_Led_Off(LED_RED);
        if(keyboard_timer==0) {
          keyboard_timer=KEY_REPEAT_MS;
          p_Basic_check_Keyboard();
        }
      }
      else {
        UB_Led_On(LED_RED);
      }

      //---------------------------------------------------
      // UART bearbeiten
      //---------------------------------------------------
      if(UB_Uart_ReceiveString(COM1,line_buf)==RX_READY) {
        // wenn etwas per UART empfangen wurde
        p_Basic_show_UARTLine();
      }

      //---------------------------------------------------
      // Cursor bearbeiten (nur wenn noch im Editor-Mode)
      //---------------------------------------------------
      if((cursor_blink_timer==0) && (UB_BASIC.mode==BASIC_MODE_EDITOR)) {
        cursor_blink_timer=CURSOR_BLINK_MS;
        UB_Basic_Editor_ToggleCursor();
      }
    }
    else if(UB_BASIC.mode==BASIC_MODE_LIST) {
      //---------------------------------------------------
      // Listing-Mode
      //---------------------------------------------------

      //---------------------------------------------------
      // Tastatur bearbeiten
      //---------------------------------------------------
      if(akt_usb_status==USB_HID_KEYBOARD_CONNECTED) {
        if(keyboard_timer==0) {
          keyboard_timer=KEY_REPEAT_MS;

          // eine Zeile vom Listing anzeigen
          p_Basic_show_Listing(1);

          p_Basic_check_Keyboard_LIST_DIR_Mode();
        }
      }
      else {
        // Listing auch ohne Tastatur anzeigen
        if(keyboard_timer==0) {
          keyboard_timer=KEY_REPEAT_MS;

          // eine Zeile vom Listing anzeigen
          p_Basic_show_Listing(1);
        }
      }
    }
    else if(UB_BASIC.mode==BASIC_MODE_DIR) {
      //---------------------------------------------------
      // Directory-Mode
      //---------------------------------------------------

      //---------------------------------------------------
      // Tastatur bearbeiten
      //---------------------------------------------------
      if(akt_usb_status==USB_HID_KEYBOARD_CONNECTED) {
        if(keyboard_timer==0) {
          keyboard_timer=KEY_REPEAT_MS;

          // eine Zeile vom Directory anzeigen
          p_Basic_show_Directory(1);

          p_Basic_check_Keyboard_LIST_DIR_Mode();
        }
      }
      else {
        // Directory auch ohne Tastatur anzeigen
        if(keyboard_timer==0) {
          keyboard_timer=KEY_REPEAT_MS;

          // eine Zeile vom Directory anzeigen
          p_Basic_show_Directory(1);
        }
      }
    }
    else if(UB_BASIC.mode==BASIC_MODE_RUN_START) {
      //---------------------------------------------------
      // Interpreter-Mode (aktiv)
      //---------------------------------------------------

      if(!ubasic_finished()) {
        // Ein Token vom Interpreter abarbeiten
        ubasic_run();
      }
      else {
        // Programm ENDE oder ERROR
        UB_BASIC.mode=BASIC_MODE_RUN_STOP;
      }

      //---------------------------------------------------
      // Tastatur/Touch bearbeiten
      //---------------------------------------------------
      if(keyboard_timer==0) {
        keyboard_timer=KEY_REPEAT_MS;
        // Keyboard
        if(akt_usb_status==USB_HID_KEYBOARD_CONNECTED) {
          p_Basic_check_Keyboard_RUN_Mode(0);
        }
        else {
          INTERPRETER.key_nr1=0;
          INTERPRETER.key_nr2=0;
          INTERPRETER.key_shift=0;
          INTERPRETER.chr_nr1=0;
          INTERPRETER.chr_nr2=0;
        }
        // Touch
        if(UB_Touch_Read()==SUCCESS) {
          INTERPRETER.touch_status=Touch_Data.status;
          INTERPRETER.touch_xp=Touch_Data.yp;
          INTERPRETER.touch_yp=(LCD_MAXX-Touch_Data.xp-1);
        }
        else {
          INTERPRETER.touch_status=TOUCH_RELEASED;
          INTERPRETER.touch_xp=0;
          INTERPRETER.touch_yp=0;
        }
      }

      //---------------------------------------------------
      // UART bearbeiten
      //---------------------------------------------------
      if(UB_Uart_ReceiveString(COM1,line_buf)==RX_READY) {
        // wenn etwas per UART empfangen wurde
        if(p_Basic_check_String(0)==1) { // ESC
          UB_BASIC.mode=BASIC_MODE_RUN_STOP;
          if(UB_BASIC.terminal==0) UB_Uart_SendString(COM1,"OK.",CRLF);
        }
        else if(p_Basic_check_String(1)==1) { // EXIT
          UB_BASIC.mode=BASIC_MODE_RUN_STOP;
          if(UB_BASIC.terminal==0) UB_Uart_SendString(COM1,"OK.",CRLF);
          UB_BASIC.source=2; // sofort beenden
        }
        else {
          // string kopieren
          UB_Basic_Interpreter_COPY_RXString(line_buf);
        }
      }
    }
    else if(UB_BASIC.mode==BASIC_MODE_RUN_STOP) {
      //---------------------------------------------------
      // Interpreter-Mode (beendet)
      //---------------------------------------------------

      // LED_GRUEN = Basic-Programm beendet
      UB_Led_On(LED_GREEN);

      //---------------------------------------------------
      if(UB_BASIC.source==2) {
        // automatisch wieder auf Editor-Mode schalten
        UB_BASIC.mode=BASIC_MODE_EDITOR;
        // Basic-Screen wieder herstellen
        UB_Basic_Editor_UpdateSreen();
        // check ob ok oder Fehler
        if(ubasic_failed()) {
          p_Basic_show_Fail();
        }
        else {
          p_Basic_show_Ok();
        }
        UB_Led_Off(LED_GREEN);
      }
      UB_BASIC.source=0;

      //---------------------------------------------------
      // Tastatur bearbeiten
      //---------------------------------------------------
      if(akt_usb_status==USB_HID_KEYBOARD_CONNECTED) {
        if(keyboard_timer==0) {
          keyboard_timer=KEY_REPEAT_MS;

          p_Basic_check_Keyboard_RUN_Mode(1);

          // check ob taste gedrueckt wurde
          if(UB_BASIC.mode==BASIC_MODE_EDITOR) {
            // Basic-Screen wieder herstellen
            UB_Basic_Editor_UpdateSreen();
            // check ob ok oder Fehler
            if(ubasic_failed()) {
              p_Basic_show_Fail();
            }
            else {
              p_Basic_show_Ok();
            }   
            UB_Led_Off(LED_GREEN);         
          }
        }
      }

      //---------------------------------------------------
      // UART bearbeiten
      //---------------------------------------------------
      if(UB_Uart_ReceiveString(COM1,line_buf)==RX_READY) {
        // wenn etwas per UART empfangen wurde
        if((p_Basic_check_String(0)==1) || (p_Basic_check_String(1)==1)) { // ESC oder EXIT
          UB_BASIC.mode=BASIC_MODE_EDITOR;
          if(UB_BASIC.terminal==0) UB_Uart_SendString(COM1,"OK.",CRLF);
          // Basic-Screen wieder herstellen
          UB_Basic_Editor_UpdateSreen();
          // check ob ok oder Fehler
          if(ubasic_failed()) {
            p_Basic_show_Fail();
          }
          else {
            p_Basic_show_Ok();
          }
          UB_Led_Off(LED_GREEN);
        }
      }
    }
    else if(UB_BASIC.mode==BASIC_MODE_LOAD) {
      //---------------------------------------------------
      // Load per UART Mode
      //---------------------------------------------------

      // warten bis Programm komplett uebertragen ist
      p_Basic_load_From_UART(1);
    }
  }
}


//--------------------------------------------------------------
// interne Funktion
// schnelle Funktion (ohne strcmp)
// um einen String zu testen
//--------------------------------------------------------------
uint8_t p_Basic_check_String(uint8_t mode)
{
  if(mode==0) {
    // check auf "ESC"
    if((line_buf[0]!='E') && (line_buf[0]!='e')) return(0);
    if((line_buf[1]!='S') && (line_buf[1]!='s')) return(0);
    if((line_buf[2]!='C') && (line_buf[2]!='c')) return(0);
  }
  else if(mode==1) {
    // check auf "EXIT"
    if((line_buf[0]!='E') && (line_buf[0]!='e')) return(0);
    if((line_buf[1]!='X') && (line_buf[1]!='x')) return(0);
    if((line_buf[2]!='I') && (line_buf[2]!='i')) return(0);
    if((line_buf[3]!='T') && (line_buf[3]!='t')) return(0);
  }

  return(1);
}


//--------------------------------------------------------------
// refresh vom LCD
// (wird zyklisch vom Timer aufgerufen)
// Durchlaufzeit ca. 1.1ms
//--------------------------------------------------------------
void UB_Basic_RefreshLCD(void)
{
  //----------------------------
  // Screen vom internen RAM ins
  // SD-RAM per DMA2D kopieren
  //----------------------------
  UB_Graphic2D_CopyBasicLCDBuffer();

  //----------------------------
  // alle Sprites kopieren
  //----------------------------
  if((UB_BASIC.mode==BASIC_MODE_RUN_START) || (UB_BASIC.mode==BASIC_MODE_RUN_STOP)) {
    UB_Graphic2D_DrawBasicSprites();
  }

  //----------------------------
  // neuen Screen anzeigen
  //----------------------------
  UB_LCD_Refresh();
}


//--------------------------------------------------------------
// interne Funktion
// init aller Hardware-Module
//--------------------------------------------------------------
void p_Basic_init_HW(void)
{

  // init vom USB (als USB-HID-Host)
  UB_USB_HID_HOST_Init();

  // init der Systick
  UB_Systick_Init();

  // init der LEDs
  UB_Led_Init();

  // init vom UART
  UB_Uart_Init(COM1,115200);

  // init der SD-Karte
  UB_Fatfs_Init();

  // init vom LCD
  #ifdef USE_VGA_SCREEN // Global define
    // init vom LCD im VGA-Mode
    UB_LCD_InitVGA();
  #else
    // init vom internen LCD
    UB_LCD_Init();
  #endif
  UB_LCD_SetMode(LANDSCAPE);
  UB_LCD_LayerInit_Fullscreen();
  // auf Hintergrund schalten
  UB_LCD_SetLayer_1();
  // Hintergrund komplett mit einer Farbe fuellen
  UB_LCD_FillLayer(RGB_COL_BLUE);
  // auf Vordergrund schalten
  UB_LCD_SetLayer_2();
  // Vordergrund komplett mit einer Farbe fuellen
  UB_LCD_FillLayer(RGB_COL_GREEN);

  // init vom Touch
  UB_Touch_Init();
}


//--------------------------------------------------------------
// interne Funktion
// init aller Software-Module
// mode : 0=nach power on
//        1=im Betrieb
//--------------------------------------------------------------
void p_Basic_init_SW(uint8_t mode)
{
  // init aller Variabeln
  if(mode==0) {
    UB_BASIC.mode=BASIC_MODE_STARTUP;
  }
  else {
    UB_BASIC.mode=BASIC_MODE_EDITOR;
  }
  UB_BASIC.source=0;
  UB_BASIC.lw=BASIC_LW_FLASH;
  UB_BASIC.terminal=0;
  UB_BASIC.key_repeat=0;
  UB_BASIC.prg_size=0;
  UB_BASIC.free_ram=STM_BASIC_RAM_SIZE;
  UB_BASIC.akt_dir_nr=0;
}


//--------------------------------------------------------------
// interne Funktion
// check der Tastatur und bearbeiten der Eingabe
// (im Editor-Mode)
//--------------------------------------------------------------
void p_Basic_check_Keyboard(void)
{
  uint8_t key_anz,akt_shift=0;
  uint8_t key1_nr=0,key2_nr=0;
  uint8_t ascii1_value=0,ascii2_value=0;
  uint8_t key_ok=0;
  uint8_t shift_mode=0;
  static uint8_t old_key_anz=0, old_key_nr=0;

  // check wieviele Tasten gedrueckt sind
  key_anz=UB_USB_HID_HOST_GetKeyAnz();
  if(key_anz>0) {
    //------------------------------------------
    // mindestens eine Taste ist gedrueckt
    //------------------------------------------
    key1_nr=USB_KEY_DATA.akt_key1;
    ascii1_value=USB_KEY_DATA.ascii_key1;
    if(key_anz==2) {
      // zweit Tasten sind gedrueckt
      key2_nr=USB_KEY_DATA.akt_key2;
      ascii2_value=USB_KEY_DATA.ascii_key2;
    }
    akt_shift=USB_KEY_DATA.akt_shift;

    //------------------------------------------
    // check auf Tastenveraenderung
    //------------------------------------------
    if(key_anz!=old_key_anz) {
      // Tastenanzahl hat sich geaendert
      if((key_anz==2) && (old_key_anz==0)) {
        // zwei Tasten schnell auf einmal gedrueckt
        // (beide Tasten nacheinander abarbeiten)
        // hier nichts zu tun
      }
      else if(key_anz==2) {
        // zweite Taste wurde gedrueckt
        // (nur die zuletzt gedrueckte abarbeiten)
        ascii1_value=ascii2_value;ascii2_value=0;
        key1_nr=key2_nr;key2_nr=0;

        // tastatur repeat zeit zuruecksetzen
        UB_BASIC.key_repeat=0;
      }
      else {
        // nur eine Taste ist gedrueckt
        // (diese Taste abarbeiten)
        ascii2_value=0;key2_nr=0;
        old_key_nr=key1_nr;
      }
      old_key_anz=key_anz;
    }
    else {
      // Tastenanzahl hat sich nicht geaendert
      if(key_anz==1) {
        // nur eine Taste ist gedrueckt
        // (diese Taste abarbeiten)
        ascii2_value=0;key2_nr=0;
        // check ob die Taste gewechselt wurde
        if(key1_nr!=old_key_nr) {
          // zwei tasten wurden schnell gewechelt
          old_key_nr=key1_nr;
          // tastatur repeat zeit zuruecksetzen
          UB_BASIC.key_repeat=0;
        }
      }
      else {
        // zwei Tasten sind gedrueckt
        // (nur die zuletzt gedrueckte abarbeiten)
        ascii1_value=ascii2_value;ascii2_value=0;
        key1_nr=key2_nr;key2_nr=0;
      }
    }

    //------------------------------------------
    // Tastatur repeat counter
    // (kleine Pause nach dem ersten Tastendruck)
    //------------------------------------------
    key_ok=0;
    if(UB_BASIC.key_repeat<KEY_DELAY) {
      UB_BASIC.key_repeat++;
      if(UB_BASIC.key_repeat==1) key_ok=1;
    }
    else {
      key_ok=1;
    }

    //------------------------------------------
    // check ob Taste abgearbeitet werden soll
    //------------------------------------------
    if(key_ok==1) {

      // check ob ALT oder STRG gedrueckt
      if((akt_shift&0x1C)!=0) {
        // STRG oder ALT ist gedrueckt
        if((akt_shift&0x01)!=0) shift_mode|=0x01; // SHIFT
        if((akt_shift&0x02)!=0) shift_mode|=0x01; // SHIFT
        if((akt_shift&0x04)!=0) shift_mode|=0x02; // STRG
        if((akt_shift&0x08)!=0) shift_mode|=0x04; // ALT
        if((akt_shift&0x10)!=0) shift_mode|=0x08; // ALT_GR
      }
      else {
        // check ob Taste-1 eine Ascii-Taste ist        
        if((ascii1_value>=EDITOR_FIRST_CHAR) && (ascii1_value<=EDITOR_LAST_CHAR)) {
          // Ascii-Code von Taste-1 anzeigen
          UB_Basic_Editor_WriteChar(ascii1_value);
          key_ok=2;
        }
        // check ob Taste-2 eine Ascii-Taste ist
        if((ascii2_value>=EDITOR_FIRST_CHAR) && (ascii2_value<=EDITOR_LAST_CHAR)) {
          // Ascii-Code von Taste-2 anzeigen
          UB_Basic_Editor_WriteChar(ascii2_value);
          key_ok=2;
        }
      }

      //-------------------------------------------------------
      // falls keine Ascii-Taste -> check auf Sondertaste
      //-------------------------------------------------------
      if(key_ok==1) {
        if(shift_mode==0x00) {
          //------------------------------------------
          // 0 = kein "SHIFT","ALT","STRG","ALT_GR"
          //------------------------------------------
          if(key1_nr==15) UB_Basic_Editor_BackspaceChar();                    // BackSpace
          else if(key1_nr==76) UB_Basic_Editor_DeleteChar();                  // Delete
          else if(key1_nr==75) UB_Basic_Editor_InsertChar(EDITOR_SPACE_CHAR); // Einfuegen
          else if(key1_nr==83) UB_Basic_Editor_MoveCursor(MOVE_CUR_UP);       // CursorUp
          else if(key1_nr==84) UB_Basic_Editor_MoveCursor(MOVE_CUR_DOWN);     // CursorDown
          else if(key1_nr==79) UB_Basic_Editor_MoveCursor(MOVE_CUR_LEFT);     // CursorLeft
          else if(key1_nr==89) UB_Basic_Editor_MoveCursor(MOVE_CUR_RIGHT);    // CursorRight
          else if(key1_nr==80) UB_Basic_Editor_MoveCursor(MOVE_CUR_X0);       // Pos1
          else if(key1_nr==81) UB_Basic_Editor_MoveCursor(MOVE_CUR_LASTCHAR); // Ende
          else if(key1_nr==43) p_Basic_line_Interpreter(0);                   // Return
          else if(key1_nr==120) UB_Basic_Editor_ClearSreen();                 // F9
          else if(key1_nr==123) UB_Basic_Editor_ChangeFont();                 // F12

          // Cursor auch mit zweiter Taste moeglich
          if(key_anz==2) {
            if(key2_nr==83) UB_Basic_Editor_MoveCursor(MOVE_CUR_UP);          // CursorUp
            else if(key2_nr==84) UB_Basic_Editor_MoveCursor(MOVE_CUR_DOWN);   // CursorDown
            else if(key2_nr==79) UB_Basic_Editor_MoveCursor(MOVE_CUR_LEFT);   // CursorLeft
            else if(key2_nr==89) UB_Basic_Editor_MoveCursor(MOVE_CUR_RIGHT);  // CursorRight
          }
        }
        else if(shift_mode==0x01) {
          //------------------------------------------
          // 1 = nur "SHIFT"
          //------------------------------------------
        }
        else if(shift_mode==0x02) {
          //------------------------------------------
          // 2 = nur "STRG"
          //------------------------------------------
          if(key1_nr==2) UB_Basic_Editor_SetVGColor(0);       // STRG+"1"
          else if(key1_nr==3) UB_Basic_Editor_SetVGColor(1);  // STRG+"2"
          else if(key1_nr==4) UB_Basic_Editor_SetVGColor(2);  // STRG+"3"
          else if(key1_nr==5) UB_Basic_Editor_SetVGColor(3);  // STRG+"4"
          else if(key1_nr==6) UB_Basic_Editor_SetVGColor(4);  // STRG+"5"
          else if(key1_nr==7) UB_Basic_Editor_SetVGColor(5);  // STRG+"6"
          else if(key1_nr==8) UB_Basic_Editor_SetVGColor(6);  // STRG+"7"
          else if(key1_nr==9) UB_Basic_Editor_SetVGColor(7);  // STRG+"8"
        }
        else if(shift_mode==0x03) {
          //------------------------------------------
          // 3 = "STRG"+"SHIFT"
          //------------------------------------------
          if(key1_nr==2) UB_Basic_Editor_SetVGColor(8);       // STRG+SHIFT+"1"
          else if(key1_nr==3) UB_Basic_Editor_SetVGColor(9);  // STRG+SHIFT+"2"
          else if(key1_nr==4) UB_Basic_Editor_SetVGColor(10); // STRG+SHIFT+"3"
          else if(key1_nr==5) UB_Basic_Editor_SetVGColor(11); // STRG+SHIFT+"4"
          else if(key1_nr==6) UB_Basic_Editor_SetVGColor(12); // STRG+SHIFT+"5"
          else if(key1_nr==7) UB_Basic_Editor_SetVGColor(13); // STRG+SHIFT+"6"
          else if(key1_nr==8) UB_Basic_Editor_SetVGColor(14); // STRG+SHIFT+"7"
          else if(key1_nr==9) UB_Basic_Editor_SetVGColor(15); // STRG+SHIFT+"8"
        }
        else if(shift_mode==0x04) {
          //------------------------------------------
          // 4 = nur "ALT"
          //------------------------------------------
          if(key1_nr==2) UB_Basic_Editor_SetBGColor(0);       // ALT+"1"
          else if(key1_nr==3) UB_Basic_Editor_SetBGColor(1);  // ALT+"2"
          else if(key1_nr==4) UB_Basic_Editor_SetBGColor(2);  // ALT+"3"
          else if(key1_nr==5) UB_Basic_Editor_SetBGColor(3);  // ALT+"4"
          else if(key1_nr==6) UB_Basic_Editor_SetBGColor(4);  // ALT+"5"
          else if(key1_nr==7) UB_Basic_Editor_SetBGColor(5);  // ALT+"6"
          else if(key1_nr==8) UB_Basic_Editor_SetBGColor(6);  // ALT+"7"
          else if(key1_nr==9) UB_Basic_Editor_SetBGColor(7);  // ALT+"8"
          else if(key1_nr==112) p_Basic_send_Screen();        // ALT+F1
        }
        else if(shift_mode==0x05) {
          //------------------------------------------
          // 5 = "ALT"+"SHIFT"
          //------------------------------------------
          if(key1_nr==2) UB_Basic_Editor_SetBGColor(8);       // ALT+SHIFT+"1"
          else if(key1_nr==3) UB_Basic_Editor_SetBGColor(9);  // ALT+SHIFT+"2"
          else if(key1_nr==4) UB_Basic_Editor_SetBGColor(10); // ALT+SHIFT+"3"
          else if(key1_nr==5) UB_Basic_Editor_SetBGColor(11); // ALT+SHIFT+"4"
          else if(key1_nr==6) UB_Basic_Editor_SetBGColor(12); // ALT+SHIFT+"5"
          else if(key1_nr==7) UB_Basic_Editor_SetBGColor(13); // ALT+SHIFT+"6"
          else if(key1_nr==8) UB_Basic_Editor_SetBGColor(14); // ALT+SHIFT+"7"
          else if(key1_nr==9) UB_Basic_Editor_SetBGColor(15); // ALT+SHIFT+"8"
        }
        else if(shift_mode==0x06) {
          //------------------------------------------
          // 6 = "STRG"+"ALT"
          //------------------------------------------
          if(key1_nr==76) {                                   // STRG+ALT+ENTF
            p_Basic_init_SW(1);
            UB_Basic_Editor_Reset();
            p_Basic_show_Header();
          }
        }
        else if(shift_mode==0x08) {
          //------------------------------------------
          // 8 = nur "ALT_GR"
          //------------------------------------------
          if(key1_nr==2) UB_Basic_Editor_SetSYSColor(0);       // ALT_GR+"1"
          else if(key1_nr==3) UB_Basic_Editor_SetSYSColor(1);  // ALT_GR+"2"
          else if(key1_nr==4) UB_Basic_Editor_SetSYSColor(2);  // ALT_GR+"3"
          else if(key1_nr==5) UB_Basic_Editor_SetSYSColor(3);  // ALT_GR+"4"
          else if(key1_nr==6) UB_Basic_Editor_SetSYSColor(4);  // ALT_GR+"5"
          else if(key1_nr==7) UB_Basic_Editor_SetSYSColor(5);  // ALT_GR+"6"
          else if(key1_nr==8) UB_Basic_Editor_SetSYSColor(6);  // ALT_GR+"7"
          else if(key1_nr==9) UB_Basic_Editor_SetSYSColor(7);  // ALT_GR+"8"          
        }
        else if(shift_mode==0x09) {
          //------------------------------------------
          // 9 = "ALT_GR"+"SHIFT"
          //------------------------------------------
          if(key1_nr==2) UB_Basic_Editor_SetSYSColor(8);       // ALT_GR+SHIFT+"1"
          else if(key1_nr==3) UB_Basic_Editor_SetSYSColor(9);  // ALT_GR+SHIFT+"2"
          else if(key1_nr==4) UB_Basic_Editor_SetSYSColor(10); // ALT_GR+SHIFT+"3"
          else if(key1_nr==5) UB_Basic_Editor_SetSYSColor(11); // ALT_GR+SHIFT+"4"
          else if(key1_nr==6) UB_Basic_Editor_SetSYSColor(12); // ALT_GR+SHIFT+"5"
          else if(key1_nr==7) UB_Basic_Editor_SetSYSColor(13); // ALT_GR+SHIFT+"6"
          else if(key1_nr==8) UB_Basic_Editor_SetSYSColor(14); // ALT_GR+SHIFT+"7"
          else if(key1_nr==9) UB_Basic_Editor_SetSYSColor(15); // ALT_GR+SHIFT+"8"
        }
      }
    }
  }
  else {
    //------------------------------------------
    // keine Taste ist gedrueckt
    //------------------------------------------
    UB_BASIC.key_repeat=0;
    old_key_anz=0;
    old_key_nr=0;
  }
}


//--------------------------------------------------------------
// interne Funktion
// check der Tastatur
// (im LIST- und DIR-Mode)
//--------------------------------------------------------------
void p_Basic_check_Keyboard_LIST_DIR_Mode(void)
{
  uint8_t key_anz,akt_shift,key_nr;

  // check wieviele Tasten gedrueckt sind
  key_anz=UB_USB_HID_HOST_GetKeyAnz();
  if(key_anz==0) {
    akt_shift=USB_KEY_DATA.akt_shift;
    if((akt_shift&0x04)!=0) {                // STRG
      UB_Systick_Pause_ms(500);
    }
  }
  else if(key_anz==1) {  
    key_nr=USB_KEY_DATA.akt_key1; 
    if(key_nr==110) {                        // ESC
      if(UB_BASIC.mode==BASIC_MODE_LIST) p_Basic_show_Listing(2);
      if(UB_BASIC.mode==BASIC_MODE_DIR) p_Basic_show_Directory(2); 
    }
  }
}


//--------------------------------------------------------------
// interne Funktion
// check der Tastatur
// (im RUN-Mode)
// mode : 0=aktiv, 1=beendet
//--------------------------------------------------------------
void p_Basic_check_Keyboard_RUN_Mode(uint8_t mode)
{
  uint8_t key_anz,key_nr;

  
  if(mode==0) {
    //-------------------------------------------------------
    // im aktiv-Mode kann nur per "ESC"
    // oder "ALT+F4" beendet werden
    //-------------------------------------------------------
    // check wieviele Tasten gedrueckt sind
    key_anz=UB_USB_HID_HOST_GetKeyAnz();
    // Shift speichern
    INTERPRETER.key_shift=USB_KEY_DATA.akt_shift;   
    if(key_anz>0) {      
      // Key-Nr speichern
      if(key_anz==1) { 
        INTERPRETER.key_nr1=USB_KEY_DATA.akt_key1;
        INTERPRETER.key_nr2=0;
        INTERPRETER.chr_nr1=USB_KEY_DATA.ascii_key1;
        INTERPRETER.chr_nr2=0;
      }             
      else {
        INTERPRETER.key_nr1=USB_KEY_DATA.akt_key1;
        INTERPRETER.key_nr2=USB_KEY_DATA.akt_key2;
        INTERPRETER.chr_nr1=USB_KEY_DATA.ascii_key1;
        INTERPRETER.chr_nr2=USB_KEY_DATA.ascii_key2;
      }
      key_nr=USB_KEY_DATA.akt_key1;      
      if(key_nr==110) {                       // ESC
        UB_BASIC.mode=BASIC_MODE_RUN_STOP; 

        // warte bis Taste wieder verlassen
        do {
          key_anz=UB_USB_HID_HOST_GetKeyAnz();
        }while(key_anz>0);
      }
      else if((key_nr==115) && ((USB_KEY_DATA.akt_shift&0x08)!=0)) { // ALT+F4
        // sofort verlassen
        UB_BASIC.mode=BASIC_MODE_RUN_STOP;         
      }
    }
    else {
      INTERPRETER.key_nr1=0;
      INTERPRETER.key_nr2=0;
      INTERPRETER.chr_nr1=0;
      INTERPRETER.chr_nr2=0;
    } 
  }
  else {
    //-------------------------------------------------------
    // im beendet-Mode kann mit jeder Taste
    // abgebrochen werden
    //-------------------------------------------------------
    // check wieviele Tasten gedrueckt sind
    key_anz=UB_USB_HID_HOST_GetKeyAnz();
    if(key_anz>0) {
      // check auf ALT+F1
      key_nr=USB_KEY_DATA.akt_key1;
      if((key_nr==112) && ((USB_KEY_DATA.akt_shift&0x08)!=0)) { // ALT+F1 
        p_Basic_send_Screen();  
      }
      else {
        // egal welche Taste, immer beenden
        UB_BASIC.mode=BASIC_MODE_EDITOR;

        // warte bis Taste wieder verlassen
        do {
          key_anz=UB_USB_HID_HOST_GetKeyAnz();
        }while(key_anz>0);
      }
    }
  }
}


//--------------------------------------------------------------
// interne Funktion
// wird aufgerufen, wenn "Enter" Taste gedrueckt wurde
// interpretiert eingegeben Befehl
// mode : 0=Eingabe per Keyboard
//        1=Eingabe per UART
//--------------------------------------------------------------
void p_Basic_line_Interpreter(uint8_t mode)
{
  uint8_t str_len,check,n;
  uint32_t line_nr;

  // kopiere Zeile in der der Cursor gerade steht
  str_len=UB_Basic_Editor_CopyLine(line_buf);
  if(str_len>0) {
    // Zeile ethaellt Zeichen
    // check welches Kommando eingegeben wurde
    UB_Basic_Interpreter_CheckString(line_buf);
    check=INTERPRETER.command;
    if(check==COMMAND_ERROR) p_Basic_show_Error();             // "Syntax error"
    else if(check==COMMAND_RESET) {                            // "reset"
      p_Basic_init_SW(1);
      UB_Basic_Editor_Reset();
      p_Basic_clear_BasicRAM();
      p_Basic_show_Header();
    }
    else if(check==COMMAND_FREE) p_Basic_show_Info();          // "free"
    else if(check==COMMAND_VERSION) p_Basic_show_Version();    // "version"
    else if(check==COMMAND_NEW) {                              // "new"
      p_Basic_clear_BasicRAM();
      p_Basic_show_Ok();
    }
    else if(check==COMMAND_CLS) UB_Basic_Editor_ClearSreen();  // "cls"
    else if(check==COMMAND_TERMINAL) p_Basic_set_Terminal();   // "terminal"
    else if(check==COMMAND_LIST) p_Basic_show_Listing(0);      // "list"
    else if(check==COMMAND_LOAD) p_Basic_load_Program();       // "load"
    else if(check==COMMAND_SAVE) p_Basic_save_Program();       // "save"
    else if(check==COMMAND_PARSE) p_Basic_parse_Program();     // "parse"
    else if(check==COMMAND_RUN) p_Basic_run_Program();         // "run"
    else if(check==COMMAND_DIR) p_Basic_show_Directory(0);     // "dir"
    else if(check==COMMAND_REMOVE) p_Basic_delete_File();      // "remove"
    else if(check==COMMAND_DEVICE) p_Basic_change_Device();    // "device"
    else if(check==COMMAND_CD) p_Basic_change_Dir();           // "cd"
    else if(check==COMMAND_DELETE) {                           // "delete"
      line_nr=INTERPRETER.merker;
      UB_Basic_Interpreter_DeleteOneLine(basic_ram, line_nr);
      // Programm groesse aktuallisieren
      UB_BASIC.prg_size=INTERPRETER.prg_size;
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      if(INTERPRETER.line_nr==0) {
        UB_Basic_Editor_WriteSystemString("LINE NOT FOUND");
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"LINE NOT FOUND",CRLF);
      }
      p_Basic_show_Ok();
    }
    else if(check==COMMAND_CODELINE) {                         // Codeline e.g. "10 a=5"
      line_nr=INTERPRETER.merker;
      UB_Basic_Interpreter_InsertOneLine(basic_ram, line_nr, line_buf);
      // Programm groesse aktuallisieren
      UB_BASIC.prg_size=INTERPRETER.prg_size;
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    }
    else if((check==COMMAND_BASIC_KEYWORD) || (check==COMMAND_BASIC_KEYWORD_AUTOESC)) {
      // zeilen-nummer '0' einfuegen
      inline_interpr[0]='0';
      inline_interpr[1]=' ';
      // zeile kopieren
      for(n=0;n<str_len;n++) {
        inline_interpr[n+2]=line_buf[n];
      }
      // endekennung hinzufuegen
      inline_interpr[n+2]=0x0A;
      inline_interpr[n+3]=0x00;
      // Zeile starten
      if(check==COMMAND_BASIC_KEYWORD) UB_BASIC.source=1;
      if(check==COMMAND_BASIC_KEYWORD_AUTOESC) UB_BASIC.source=2;
      p_Basic_run_Program();
    }
    // ausgabe an UART
    if((mode==1) && (UB_BASIC.terminal==0)) {
      if(INTERPRETER.command==COMMAND_ERROR) {
        UB_Uart_SendString(COM1,"UNKNOWN.",CRLF);
      }
      else {
        UB_Uart_SendString(COM1,"OK.",CRLF);
      }
    }
  }
  else {
    // Zeile war leer
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  }
}


//--------------------------------------------------------------
// interne Funktion
// Header vom Basic anzeigen
//--------------------------------------------------------------
void p_Basic_show_Header(void)
{
  UB_Basic_Editor_SetCursor(4,1);
  sprintf(line_buf,"**** STM32F429 BASIC V:%s ****",VERSION_STRING);
  UB_Basic_Editor_WriteString(line_buf);

  UB_Basic_Editor_SetCursor(1,3);
  sprintf(line_buf,"%dK RAM SYSTEM %d BYTES FREE",(int)(STM_BASIC_RAM_KB),(int)(UB_BASIC.free_ram));
  UB_Basic_Editor_WriteString(line_buf);
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);

  sprintf(line_buf,"**** STM32F429 BASIC V:%s ****",VERSION_STRING);
  UB_Uart_SendString(COM1,line_buf,CRLF);
  sprintf(line_buf,"%dK RAM SYSTEM %d BYTES FREE",(int)(STM_BASIC_RAM_KB),(int)(UB_BASIC.free_ram));
  UB_Uart_SendString(COM1,line_buf,CRLF);
  UB_Uart_SendString(COM1," ",CRLF);

  p_Basic_show_Copyright();
  p_Basic_show_Ok();
}


//--------------------------------------------------------------
// interne Funktion
// Version vom Basic anzeigen
//--------------------------------------------------------------
void p_Basic_show_Version(void)
{
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  sprintf(line_buf,"Version : %s / %s",VERSION_STRING,DATE_STRING);
  UB_Basic_Editor_WriteSystemString(line_buf);
  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);
  p_Basic_show_Ok();
}

//--------------------------------------------------------------
// interne Funktion
// zeigt das Copyright an
//--------------------------------------------------------------
void p_Basic_show_Copyright(void)
{
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString("COPYRIGHT (c) 2014, Uwe Becker"); 
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteString("EMAIL : mc-4u@t-online.de");
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteString("WEB   : www.mikrocontroller-4u.de");
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);

  UB_Uart_SendString(COM1,"COPYRIGHT (c) 2014, Uwe Becker",CRLF);
  UB_Uart_SendString(COM1,"EMAIL : mc-4u@t-online.de",CRLF);
  UB_Uart_SendString(COM1,"WEB   : www.mikrocontroller-4u.de",CRLF);
  UB_Uart_SendString(COM1," ",CRLF);
  UB_Uart_SendString(COM1,"sent 'TERMINAL 1' to aktivate the terminal",CRLF);
  UB_Uart_SendString(COM1," ",CRLF);
}


//--------------------------------------------------------------
// interne Funktion
// zeigt aktuellen Speicherplatz an
//--------------------------------------------------------------
void p_Basic_show_Info(void)
{
  uint32_t max,used,free;

  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  sprintf(line_buf,"BASIC-RAM  : %d USED %d FREE",(int)(UB_BASIC.prg_size),(int)(UB_BASIC.free_ram));
  UB_Basic_Editor_WriteSystemString(line_buf);
  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);

  max=ubasic_getramstatus(0);
  used=ubasic_getramstatus(1);
  free=max-used;
  sprintf(line_buf,"INT-ARRAY  : %d USED %d FREE",(int)(used),(int)(free));
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString(line_buf);
  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);

  max=ubasic_getramstatus(2);
  used=ubasic_getramstatus(3);
  free=max-used;
  sprintf(line_buf,"STR-ARRAY  : %d USED %d FREE",(int)(used),(int)(free));
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString(line_buf);
  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);

  max=ubasic_getramstatus(4);
  used=ubasic_getramstatus(5);
  free=max-used;
  sprintf(line_buf,"DATA-ARRAY : %d USED %d FREE",(int)(used),(int)(free));
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString(line_buf);
  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);

  p_Basic_show_Ok();
}


//--------------------------------------------------------------
// interne Funktion
// OK Meldung anzeigen
//--------------------------------------------------------------
void p_Basic_show_Ok(void)
{
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString("READY.");
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);

  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"READY.",CRLF);
}


//--------------------------------------------------------------
// interne Funktion
// OK Meldung anzeigen
//--------------------------------------------------------------
void p_Basic_show_Ok2(void)
{
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString("READY.");
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
}


//--------------------------------------------------------------
// interne Funktion
// Error Meldung anzeigen
//--------------------------------------------------------------
void p_Basic_show_Error(void)
{
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString("?SYNTAX ERROR");

  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"?SYNTAX ERROR",CRLF);

  p_Basic_show_Ok();
}


//--------------------------------------------------------------
// interne Funktion
// Fehler Meldung anzeigen
//--------------------------------------------------------------
void p_Basic_show_Fail(void)
{
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  if(ubasic_failed()==1) sprintf(line_buf,"?PRE-PARSE ERROR IN %d",ubasic_linenum());
  if(ubasic_failed()==2) sprintf(line_buf,"?UNEXPECTED TOKEN IN %d",ubasic_linenum());
  if(ubasic_failed()==3) sprintf(line_buf,"?INVALID KEYWORD IN %d",ubasic_linenum());
  if(ubasic_failed()==4) sprintf(line_buf,"?INVALID DATA IN %d",ubasic_linenum());
  if(ubasic_failed()==5) sprintf(line_buf,"?OVERFLOW ERROR IN %d",ubasic_linenum());
  if(ubasic_failed()==6) sprintf(line_buf,"?OUT OF MEMORY ERROR IN %d",ubasic_linenum());
  if(ubasic_failed()>6) sprintf(line_buf,"?ERROR %d IN %d",ubasic_failed(),ubasic_linenum());
  UB_Basic_Editor_WriteSystemString(line_buf);

  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);

  p_Basic_show_Ok();
}


//--------------------------------------------------------------
// interne Funktion
// Zeile anzeigen, die per UART empfangen wurde
// und line interpreter aufrufen
//--------------------------------------------------------------
void p_Basic_show_UARTLine(void)
{
  UB_Basic_Editor_WriteString(line_buf);

  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);

  p_Basic_line_Interpreter(1);
}


//--------------------------------------------------------------
// interne Funktion
// Basic RAM loeschen
//--------------------------------------------------------------
void p_Basic_clear_BasicRAM(void)
{
  basic_ram[0]=0x00;

  UB_BASIC.prg_size=1;
  UB_BASIC.free_ram=STM_BASIC_RAM_SIZE-UB_BASIC.prg_size;
}


//--------------------------------------------------------------
// interne Funktion
// Terminal ein/aus schalten
// Device steht in "INTERPRETER.merker"
//--------------------------------------------------------------
void p_Basic_set_Terminal(void)
{
  if(INTERPRETER.merker==0) {
    UB_BASIC.terminal=0;
    p_Basic_show_Ok();
  }
  else if(INTERPRETER.merker==1) {
    UB_BASIC.terminal=1;
    p_Basic_show_Ok();
  }
  else p_Basic_show_Error();
}


//--------------------------------------------------------------
// interne Funktion
// Listing anzeigen
// mode : 0=start, 1=weiter, 2=Break
//--------------------------------------------------------------
void p_Basic_show_Listing(uint8_t mode)
{
  static uint32_t ram_adr=0, stop_adr;
  uint8_t wert,ok,laenge;

  if(mode==0) {
    // start vom Listing
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    if(UB_BASIC.prg_size>1) {
      UB_BASIC.mode=BASIC_MODE_LIST;

      ram_adr=0;
      stop_adr=UB_BASIC.prg_size;
      if((INTERPRETER.merker&0x01)!=0) {
        // list ab zeilen-nr : INTERPRETER.z1
        ram_adr=UB_Basic_Interpreter_GetLineAdr(basic_ram, INTERPRETER.z1, 0);
        if((INTERPRETER.merker&0x02)==0) {
          // list von nur einer zeilen-nr
          stop_adr=UB_Basic_Interpreter_GetLineAdr(basic_ram, INTERPRETER.z1, 1);
          if(stop_adr==0) stop_adr=UB_BASIC.prg_size;
        }
      }
      if((INTERPRETER.merker&0x04)!=0) {
        // list bis zeilen-nr : INTERPRETER.z2
        stop_adr=UB_Basic_Interpreter_GetLineAdr(basic_ram, INTERPRETER.z2, 1);
        if(stop_adr==0) stop_adr=UB_BASIC.prg_size;
      }

      mode=1;
    }
    else {
      // kein Program im Speicher
      UB_Basic_Editor_WriteSystemString("BASIC RAM EMPTY");
      if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"BASIC RAM EMPTY",CRLF);
      p_Basic_show_Ok();
    }
  }

  // eine Zeile anzeigen
  if(mode==1) {
    ok=0;
    do {
      // ein Zeichen vom RAM auslesen
      wert=basic_ram[ram_adr];
      ram_adr++;
      if(wert==EDITOR_LINEFEED) {
        // ende der Zeile erreicht
        UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"",CRLF);
        ok=1;
      }
      else if((wert==0x00) || (ram_adr>=stop_adr)) {
        // ende vom Programm erreicht
        UB_BASIC.mode=BASIC_MODE_EDITOR;
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"",CRLF);
        p_Basic_show_Ok();
        ok=1;
      }
      else if(wert==0x01) {
        // keyword anzeigen
        wert=basic_ram[ram_adr];
        // keyword auslesen
        laenge=ubasic_getkeyword(wert, line_buf); 
        // keyword komplett anzeigen
        UB_Basic_Editor_WriteSystemString(line_buf);
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,NONE);
        // adresse aktuallisieren
        ram_adr+=laenge-1;
      }
      else {
        // das Zeichen anzeigen
        UB_Basic_Editor_WriteChar(wert);
        if(UB_BASIC.terminal==1) UB_Uart_SendByte(COM1,wert);
      }
    }while(ok==0);
  }

  if(mode==2) { 
    // Listing abbrechen
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_BASIC.mode=BASIC_MODE_EDITOR;
    UB_Basic_Editor_WriteSystemString("BREAK");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"BREAK",CRLF);
    p_Basic_show_Ok();
  }
}


//--------------------------------------------------------------
// interne Funktion
// anzeigen vom directory (FLASH/SD)
// mode : 0=start, 1=weiter, 2=Break
//--------------------------------------------------------------
void p_Basic_show_Directory(uint8_t mode)
{
  FRESULT rc;
  static DIR dir;
  FILINFO fno;
  uint8_t ok;

  if(UB_BASIC.lw==BASIC_LW_UART) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("CANT SHOW DIR FROM UART");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"CANT SHOW DIR FROM UART",CRLF);
    p_Basic_show_Ok();     
  }
  else if(UB_BASIC.lw==BASIC_LW_FLASH) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("[FLASH]");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"[FLASH]",CRLF);
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    for(ok=0;ok<BASIC_ROM_ITEMS;ok++) {
      if(UB_BASIC.akt_dir_nr==BASIC_ROM[ok].akt_dir_nr) {
    	if(BASIC_ROM[ok].file_typ==0) {
          // Directory
          UB_Basic_Editor_WriteSystemString("<DIR>");
          UB_Basic_Editor_WriteString(BASIC_ROM[ok].filename);
          UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
          if(UB_BASIC.terminal==1) {
            UB_Uart_SendString(COM1,"<DIR>",NONE);
            UB_Uart_SendString(COM1,BASIC_ROM[ok].filename,CRLF);
          }
    	}
    	else {
          // File
          UB_Basic_Editor_WriteString(BASIC_ROM[ok].filename);
          UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
          if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,BASIC_ROM[ok].filename,CRLF);
    	}
      }
    }
 
    p_Basic_show_Ok();
  }
  else if(UB_BASIC.lw==BASIC_LW_SD) {
    if(mode==0) {
      // start vom Directory
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      if(UB_Fatfs_Mount(MMC_0)==FATFS_OK) {
    	if((rc=f_opendir(&dir, "0:")) != FR_OK) {
          UB_Basic_Editor_WriteSystemString("DRIVE EMPTY");
          if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"DRIVE EMPTY",CRLF);
          UB_Fatfs_UnMount(MMC_0);
          p_Basic_show_Ok();
    	}
    	else {
    	  UB_Basic_Editor_WriteSystemString("[SD-CARD]");
    	  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    	  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"[SD-CARD]",CRLF);
          UB_BASIC.mode=BASIC_MODE_DIR;
          mode=1;
    	}
      }
      else {
        // Fehler beim oeffnen
        UB_Basic_Editor_WriteSystemString("SD-CARD ERROR");
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"SD-CARD ERROR",CRLF);
        p_Basic_show_Ok();
      }
    }

    // eine Zeile anzeigen
    if(mode==1) {
      ok=0;
      do {
        // einen Directory eintrag einlesen
        rc = f_readdir(&dir, &fno);
        if(rc != FR_OK || fno.fname[0] == 0) {
          // ende vom Directory erreicht
          ok=1;
          UB_Fatfs_UnMount(MMC_0);
          UB_BASIC.mode=BASIC_MODE_EDITOR;
          p_Basic_show_Ok();        
        }
        else {
          if(fno.fname[0] != '.') {
            if(fno.fattrib & AM_DIR) {            
              UB_Basic_Editor_WriteSystemString("<DIR>");
              UB_Basic_Editor_WriteString(fno.fname);
              if(UB_BASIC.terminal==1) {
                UB_Uart_SendString(COM1,"<DIR>",NONE);
                UB_Uart_SendString(COM1,fno.fname,CRLF);
              }
            }
            else {
              UB_Basic_Editor_WriteString(fno.fname);
              if(UB_BASIC.terminal==1) {
                UB_Uart_SendString(COM1,fno.fname,CRLF);
              }
            } 
            UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
            ok=1;        
          }
        }
      }while(ok==0);
    }

    if(mode==2) { 
      // Directory abbrechen
      UB_Fatfs_UnMount(MMC_0);
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      UB_BASIC.mode=BASIC_MODE_EDITOR;
      UB_Basic_Editor_WriteSystemString("BREAK");
      if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"BREAK",CRLF);
      p_Basic_show_Ok();
    }
  }
  else if(UB_BASIC.lw==BASIC_LW_USB) {
    p_Basic_show_Error();
  }
  else p_Basic_show_Error();
}


//--------------------------------------------------------------
// interne Funktion
// aendert aktives direktory
// Direktory steht in "INTERPRETER.filename"
//--------------------------------------------------------------
void p_Basic_change_Dir(void)
{
  char wert;
  uint32_t n;

  if(UB_BASIC.lw==BASIC_LW_UART) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("CANT CANGE DIR FROM UART");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"CANT CANGE DIR FROM UART",CRLF);
    p_Basic_show_Ok();
  }
  else if(UB_BASIC.lw==BASIC_LW_FLASH) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("CHANGE DIRECTORY : >");
    UB_Basic_Editor_WriteString(INTERPRETER.filename);
    UB_Basic_Editor_WriteSystemString("<");

    // Dir vom Flash auswaehlen
    wert=0;
    for(n=0;n<BASIC_ROM_ITEMS;n++) {
      if(UB_BASIC.akt_dir_nr==BASIC_ROM[n].akt_dir_nr) {
        if(BASIC_ROM[n].file_typ==0) {
          if(strcmp(INTERPRETER.filename,BASIC_ROM[n].filename)==0) {
            // wenn Dir in Liste gefunden wurde
            wert=1;
            UB_BASIC.akt_dir_nr=BASIC_ROM[n].new_dir_nr;
            p_Basic_show_Ok();
            break;
          }
        }
      }
    }

    if(wert==0) {
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      UB_Basic_Editor_WriteSystemString("DIRECTORY NOT FOUND");
      if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"DIRECTORY NOT FOUND",CRLF);
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      return;
    }
  }
  else if(UB_BASIC.lw==BASIC_LW_SD) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("CANT CANGE DIR FROM SD-CARD");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"CANT CANGE DIR FROM SD-CARD",CRLF);
    p_Basic_show_Ok();
  }
  else if(UB_BASIC.lw==BASIC_LW_USB) {
    p_Basic_show_Error();
  }
  else p_Basic_show_Error();
}

//--------------------------------------------------------------
// interne Funktion
// aendert aktives device
// Device steht in "INTERPRETER.merker"
//--------------------------------------------------------------
void p_Basic_change_Device(void)
{
  if(INTERPRETER.merker==0) {
    UB_BASIC.lw=BASIC_LW_UART;
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("DEVICE = UART");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"DEVICE = UART",CRLF);
    p_Basic_show_Ok();
  }
  else if(INTERPRETER.merker==1) {
    UB_BASIC.lw=BASIC_LW_FLASH;
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("DEVICE = FLASH");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"DEVICE = FLASH",CRLF);
    p_Basic_show_Ok();
  }
  else if(INTERPRETER.merker==2) {
    UB_BASIC.lw=BASIC_LW_SD;
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("DEVICE = SD-CARD");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"DEVICE = SD-CARD",CRLF);
    p_Basic_show_Ok();
  }
  else if(INTERPRETER.merker==3) {
    // UB_BASIC.lw=BASIC_LW_USB;
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("USB-DRIVE NOT SUPPORTED IN THIS VERSION");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"USB-DRIVE NOT SUPPORTED IN THIS VERSION",CRLF);
    p_Basic_show_Ok();
  }  
  else {
    p_Basic_show_Error();
  }    
}


//--------------------------------------------------------------
// interne Funktion
// File loeschen (SD/USB)
//--------------------------------------------------------------
void p_Basic_delete_File(void)
{
  if(UB_BASIC.lw==BASIC_LW_UART) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("CANT REMOVE FROM UART");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"CANT REMOVE FROM UART",CRLF);
    p_Basic_show_Ok();
  }
  else if(UB_BASIC.lw==BASIC_LW_FLASH) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("CANT REMOVE FROM FLASH");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"CANT REMOVE FROM FLASH",CRLF);
    p_Basic_show_Ok();
  }
  else if(UB_BASIC.lw==BASIC_LW_SD) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("REMOVE FROM SD-CARD : >");
    UB_Basic_Editor_WriteString(INTERPRETER.filename);
    UB_Basic_Editor_WriteSystemString("<");

    if(UB_BASIC.terminal==1) {
      UB_Uart_SendString(COM1,"REMOVE FROM SD-CARD : >",NONE);
      UB_Uart_SendString(COM1,INTERPRETER.filename,NONE);
      UB_Uart_SendString(COM1,"<",CRLF);
    }

    if(UB_Fatfs_Mount(MMC_0)==FATFS_OK) {
      if(UB_Fatfs_DelFile(INTERPRETER.filename)==FATFS_OK) {
        UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
        UB_Basic_Editor_WriteSystemString("FILE REMOVED");
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"FILE REMOVED",CRLF);
      }
      else {
        UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
        UB_Basic_Editor_WriteSystemString("REMOVE ERROR");
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"REMOVE ERROR",CRLF);
      }
      UB_Fatfs_UnMount(MMC_0);
    } 
    else {
      // Fehler beim oeffnen
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      UB_Basic_Editor_WriteSystemString("SD-CARD ERROR");  
      if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"SD-CARD ERROR",CRLF);  
    } 

    p_Basic_show_Ok();
  }
  else if(UB_BASIC.lw==BASIC_LW_USB) {
    p_Basic_show_Error();
  }
  else p_Basic_show_Error();
}


//--------------------------------------------------------------
// interne Funktion
// Program laden (UART/FLASH/SD/USB)
// Filename steht in "INTERPRETER.filename"
//--------------------------------------------------------------
void p_Basic_load_Program(void)
{
  if(UB_BASIC.lw==BASIC_LW_UART) {
    p_Basic_load_From_UART(0);
  }
  else if(UB_BASIC.lw==BASIC_LW_FLASH) {
    p_Basic_load_From_FLASH();
  }
  else if(UB_BASIC.lw==BASIC_LW_SD) {
    p_Basic_load_From_SD();
  }
  else if(UB_BASIC.lw==BASIC_LW_USB) {
    p_Basic_show_Error();
  }
  else p_Basic_show_Error();
}


//--------------------------------------------------------------
// interne Funktion
// Program laden (per UART)
// mode : 0=start, 1=weiter
//--------------------------------------------------------------
void p_Basic_load_From_UART(uint8_t mode)
{
  static uint32_t timeout=0,max_timeout=0;
  static uint32_t old_prg_size=0;
  static uint32_t akt_prg_size=0;
  static uint8_t start=0;

  if(mode==0) {
    // start vom laden per UART
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("WAIT FOR DATA FROM UART");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"WAIT FOR DATA FROM UART",CRLF);
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_BASIC.mode=BASIC_MODE_LOAD;

    old_prg_size=UB_BASIC.prg_size;
    UB_BASIC.prg_size=0;
    akt_prg_size=0;
    timeout=0;
    max_timeout=UART_LOAD_TIMEOUT;
    start=0;
  }
  else {
    // uebertragung
    if(UB_BASIC.prg_size!=0) {
      // wenn Daten per UART empfangen wurden
      if(start==0) {
        // erstes Byte wurde empfangen
        UB_Led_On(LED_RED);
        start=1;
        UB_BASIC.free_ram=STM_BASIC_RAM_SIZE;
        UB_Basic_Editor_WriteSystemString("LOADING...PLEASE WAIT");
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"LOADING...PLEASE WAIT",CRLF);
        timeout=0;
        max_timeout=UART_LOAD_TIMEOUT2;
      }
      else {
        if(UB_BASIC.prg_size!=akt_prg_size) {
          // wenn neue Daten empfangen wurden
          // Timeout zuruecksetzen
          akt_prg_size=UB_BASIC.prg_size;
          timeout=0;
        }
      }
    }

    // warte auf Timeout
    timeout++;
    if(timeout>max_timeout) {
      // Timeout ist abgelaufen
      if(start==0) {
        // es wurden keine Daten empfangen
        UB_Basic_Editor_WriteSystemString("TIMEOUT ERROR");
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"TIMEOUT ERROR",CRLF);
        // alte Daten wieder herstellen
        UB_BASIC.prg_size=old_prg_size;
      }
      else {
        // alle Daten wurden empfangen
        // Endekennung pruefen
    	p_Basic_check_BasicRAM();
        // Speicherplatz ausrechnen und anzeigen
    	UB_BASIC.free_ram=STM_BASIC_RAM_SIZE-UB_BASIC.prg_size;
        UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
        sprintf(line_buf,"%d BYTES LOADED %d BYTES FREE",(int)(UB_BASIC.prg_size),(int)(UB_BASIC.free_ram));
        UB_Basic_Editor_WriteSystemString(line_buf);
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);
      }
      // auf Editor-Mode zurueckschalten
      UB_BASIC.mode=BASIC_MODE_EDITOR;
      p_Basic_show_Ok();
      UB_Led_Off(LED_RED);
    }
  }
}


//--------------------------------------------------------------
// interne Funktion
// Program laden (von Flash) und parsen
// Filename steht in "INTERPRETER.filename"
//--------------------------------------------------------------
void p_Basic_load_From_FLASH(void)
{
  uint32_t ram_adr=0,rom_adr=0,laenge=0;
  char wert;
  uint8_t n;
  const uint8_t *ptr;

  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString("LOAD FROM FLASH : >");
  UB_Basic_Editor_WriteString(INTERPRETER.filename);
  UB_Basic_Editor_WriteSystemString("<");

  if(UB_BASIC.terminal==1) {
    UB_Uart_SendString(COM1,"LOAD FROM FLASH : >",NONE);
    UB_Uart_SendString(COM1,INTERPRETER.filename,NONE);
    UB_Uart_SendString(COM1,"<",CRLF);
  }

  // Programm vom Flash auswaehlen
  wert=0;
  for(n=0;n<BASIC_ROM_ITEMS;n++) {
    if(UB_BASIC.akt_dir_nr==BASIC_ROM[n].akt_dir_nr) {
      if(BASIC_ROM[n].file_typ==1) {
        if(strcmp(INTERPRETER.filename,BASIC_ROM[n].filename)==0) {
          // wenn Filename in Liste gefunden wurde
          wert=1;
          ptr=BASIC_ROM[n].fileptr->table;
          laenge=BASIC_ROM[n].fileptr->size;
          break;
        }
      }
    }
  }

  if(wert==0) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("FILE NOT FOUND");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"FILE NOT FOUND",CRLF); 
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    return;   
  }


  UB_BASIC.free_ram=STM_BASIC_RAM_SIZE;
  UB_BASIC.prg_size=0;
  
  // flash ins Basic-RAM kopieren
  do {
    wert=ptr[rom_adr];
    if(wert!=0x0D) {
      basic_ram[ram_adr]=wert;
      ram_adr++;
    }
    rom_adr++;
  }
  while(rom_adr!=laenge);

  UB_BASIC.prg_size=ram_adr;
  // Endekennung pruefen
  p_Basic_check_BasicRAM();

  // Speicherplatz ausrechnen und anzeigen
  UB_BASIC.free_ram=STM_BASIC_RAM_SIZE-UB_BASIC.prg_size;
  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  sprintf(line_buf,"%d BYTES LOADED %d BYTES FREE",(int)(UB_BASIC.prg_size),(int)(UB_BASIC.free_ram));
  UB_Basic_Editor_WriteSystemString(line_buf);
  if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);
  // Programm gleich parsten (fuer Syntax Highlight)
  p_Basic_parse_Program();
}


//--------------------------------------------------------------
// interne Funktion
// Program laden (von SD-Karte)
// Filename steht in "INTERPRETER.filename"
//--------------------------------------------------------------
void p_Basic_load_From_SD(void)
{
  FIL myFile;
  FATFS_t check;
  char wert;
  uint32_t n,read_size;
  uint32_t ram_adr=0;

  UB_Led_On(LED_RED);

  UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
  UB_Basic_Editor_WriteSystemString("LOAD FROM SD-CARD : >");
  UB_Basic_Editor_WriteString(INTERPRETER.filename);
  UB_Basic_Editor_WriteSystemString("<");

  if(UB_BASIC.terminal==1) {
    UB_Uart_SendString(COM1,"LOAD FROM SD-CARD : >",NONE);
    UB_Uart_SendString(COM1,INTERPRETER.filename,NONE);
    UB_Uart_SendString(COM1,"<",CRLF);
  }

  if(UB_Fatfs_Mount(MMC_0)==FATFS_OK) {
    if(UB_Fatfs_OpenFile(&myFile,INTERPRETER.filename,F_RD)==FATFS_OK) {
      UB_BASIC.free_ram=STM_BASIC_RAM_SIZE;
      UB_BASIC.prg_size=0;

      do {
        check=UB_Fatfs_ReadBlock(&myFile,line_buf,85,&read_size);
        for(n=0;n<read_size;n++) {
          wert=line_buf[n];
          if(wert!=RX_END_CHR) {
            // 0x0D (13d) nicht laden
            basic_ram[ram_adr]=wert;
            ram_adr++;
          }
        }
      }while(check==FATFS_OK);
      // File schliessen
      UB_Fatfs_CloseFile(&myFile);

      UB_BASIC.prg_size=ram_adr;
      // Endekennung pruefen
      p_Basic_check_BasicRAM();
      // Speicherplatz ausrechnen und anzeigen
      UB_BASIC.free_ram=STM_BASIC_RAM_SIZE-UB_BASIC.prg_size;
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      sprintf(line_buf,"%d BYTES LOADED %d BYTES FREE",(int)(UB_BASIC.prg_size),(int)(UB_BASIC.free_ram));
      UB_Basic_Editor_WriteSystemString(line_buf);
      if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);
    }
    else {
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      UB_Basic_Editor_WriteSystemString("FILE NOT FOUND");
      if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"FILE NOT FOUND",CRLF);
    }

    UB_Fatfs_UnMount(MMC_0);
  }
  else {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("SD-CARD ERROR");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"SD-CARD ERROR",CRLF);
  }

  UB_Led_Off(LED_RED);

  p_Basic_show_Ok();
}


//--------------------------------------------------------------
// interne Funktion
// prueft Endekennung vom Basic-RAM
// Soll : <Data><Data><0x0A><0x00><EOF>
//--------------------------------------------------------------
void p_Basic_check_BasicRAM(void)
{
  uint32_t last_adr;

  last_adr=UB_BASIC.prg_size-1;

  if(basic_ram[last_adr]==0x00) {
    if(basic_ram[last_adr-1]!=0x0A) {
      basic_ram[last_adr]=0x0A;
      basic_ram[last_adr+1]=0x00;
      UB_BASIC.prg_size++;
    }
  }
  else if(basic_ram[last_adr]==0x0A) {
    basic_ram[last_adr+1]=0x00;
    UB_BASIC.prg_size++;
  }
  else {
    basic_ram[last_adr+1]=0x0A;
    basic_ram[last_adr+2]=0x00;
    UB_BASIC.prg_size+=2;
  }
}


//--------------------------------------------------------------
// interne Funktion
// Program speichern (UART/SD/USB)
// Filename steht in "INTERPRETER.filename"
//--------------------------------------------------------------
void p_Basic_save_Program(void)
{

  if(UB_BASIC.lw==BASIC_LW_UART) {
    p_Basic_save_To_UART();   
  }
  else if(UB_BASIC.lw==BASIC_LW_FLASH) {
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("CANT SAVE TO FLASH");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"CANT SAVE TO FLASH",CRLF);
    p_Basic_show_Ok();
  }  
  else if(UB_BASIC.lw==BASIC_LW_SD) {
    p_Basic_save_To_SD();
  } 
  else if(UB_BASIC.lw==BASIC_LW_USB) {
    p_Basic_show_Error();
  } 
  else p_Basic_show_Error();
}


//--------------------------------------------------------------
// interne Funktion
// Program speichern (per UART)
//--------------------------------------------------------------
void p_Basic_save_To_UART(void)
{
  uint32_t ram_adr;
  uint8_t wert,laenge;

  if(UB_BASIC.prg_size>1) {
    UB_Led_On(LED_RED);
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("SAVING TO UART...PLEASE WAIT");    

    for(ram_adr=0;ram_adr<UB_BASIC.prg_size;ram_adr++) {
      wert=basic_ram[ram_adr];
      if(wert==EDITOR_LINEFEED) {
        // Zeilenende fuer STM32F4-PC-Terminal-UB
        UB_Uart_SendByte(COM1,0x0D); // zuerst CR
        UB_Uart_SendByte(COM1,0x0A); // dann LF
      }
      else if(wert==0x01) {
        // keyword senden
        wert=basic_ram[ram_adr+1];
        // keyword auslesen
        laenge=ubasic_getkeyword(wert, line_buf);
        // keyword komplett senden
        UB_Uart_SendString(COM1,line_buf,NONE);
        // adresse aktuallisieren
        ram_adr+=laenge-1;
      }
      else {
        // ein Zeichen senden
        UB_Uart_SendByte(COM1,wert);
      }
    }
    UB_Led_Off(LED_RED);
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    sprintf(line_buf,"%d BYTES SENT",(int)(UB_BASIC.prg_size));
    UB_Basic_Editor_WriteSystemString(line_buf);
    p_Basic_show_Ok2();
  }
  else {
    // kein Program im Speicher
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("BASIC RAM EMPTY");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"BASIC RAM EMPTY",CRLF);
    p_Basic_show_Ok();
  }
}


//--------------------------------------------------------------
// interne Funktion
// Program speichern (auf SD-Karte)
// Filename steht in "INTERPRETER.filename"
//--------------------------------------------------------------
void p_Basic_save_To_SD(void)
{
  FIL myFile;
  FRESULT check;
  uint32_t ram_adr;
  UINT write_size;
  uint8_t laenge;
  char *wert;

  if(UB_BASIC.prg_size>1) {
    UB_Led_On(LED_RED);

    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("SAVE TO SD-CARD : >");
    UB_Basic_Editor_WriteString(INTERPRETER.filename);
    UB_Basic_Editor_WriteSystemString("<");

    if(UB_BASIC.terminal==1) {
      UB_Uart_SendString(COM1,"SAVE TO SD-CARD : >",NONE);
      UB_Uart_SendString(COM1,INTERPRETER.filename,NONE);
      UB_Uart_SendString(COM1,"<",CRLF);
    }

    if(UB_Fatfs_Mount(MMC_0)==FATFS_OK) {
      if(UB_Fatfs_OpenFile(&myFile,INTERPRETER.filename,F_WR_CLEAR)==FATFS_OK) {
        for(ram_adr=0;ram_adr<UB_BASIC.prg_size;ram_adr++) {
          wert=&basic_ram[ram_adr];
          if(*wert==EDITOR_LINEFEED) {
            // Zeilenende fuer File
            *wert=0x0D;
            check=f_write(&myFile,wert,1,&write_size); // zuerst CR
            *wert=0x0A;
            check=f_write(&myFile,wert,1,&write_size); // dann LF
          }
          else if(*wert==0x01) {
            // keyword anzeigen
            wert=&basic_ram[ram_adr+1];
            // keyword auslesen
            laenge=ubasic_getkeyword(*wert, line_buf);
            // keyword komplett screiben
            check=f_write(&myFile,line_buf,laenge,&write_size);           
            // adresse aktuallisieren
            ram_adr+=laenge-1;
          }
          else {
            // ein Zeichen schreiben
            check=f_write(&myFile,wert,1,&write_size);
          }
          // check ob fehler beim schreiben
          if(check!=FR_OK) break;
        }
        
        if(check==FR_OK) {
          UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
          sprintf(line_buf,"%d BYTES SAVED",(int)(UB_BASIC.prg_size));
          UB_Basic_Editor_WriteSystemString(line_buf);
          if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,line_buf,CRLF);
        }
        else {
          UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
          UB_Basic_Editor_WriteSystemString("SD-WRITE ERROR");
          if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"SD-WRITE ERROR",CRLF);
        }
        // File schliessen
        UB_Fatfs_CloseFile(&myFile);
      }
      else {
        UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
        UB_Basic_Editor_WriteSystemString("SD-WRITE ERROR");
        if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"SD-WRITE ERROR",CRLF);
      }

      UB_Fatfs_UnMount(MMC_0);
    }
    else {
      UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
      UB_Basic_Editor_WriteSystemString("SD-CARD ERROR");
      if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"SD-CARD ERROR",CRLF);
    }

    UB_Led_Off(LED_RED);
    p_Basic_show_Ok();
  }
  else {
    // kein Program im Speicher
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("BASIC RAM EMPTY");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"BASIC RAM EMPTY",CRLF);
    p_Basic_show_Ok();
  }
}


//--------------------------------------------------------------
// interne Funktion
// pre-parse vom Basic-Program
//--------------------------------------------------------------
void p_Basic_parse_Program(void)
{
  if(UB_BASIC.prg_size>1) {
    // init vom uBasic und start vom pre-parser
    ubasic_init(basic_ram,0);
    if(ubasic_failed()) {
      // fehler vom pre-parser erkannt
      p_Basic_show_Fail();
    }
    else {
      // kein Fehler      
      p_Basic_show_Ok();      
    }
  }
  else {
    // kein Basic-Program im Speicher
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("BASIC RAM EMPTY");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"BASIC RAM EMPTY",CRLF);
    p_Basic_show_Ok();
  }
}


//--------------------------------------------------------------
// interne Funktion
// Basic-Program starten
//--------------------------------------------------------------
void p_Basic_run_Program(void)
{
  uint8_t key_anz;

  if((UB_BASIC.prg_size>1) || (UB_BASIC.source!=0)) {
    //-------------------------------------
    // globale Werte vom uBasic setzen
    //-------------------------------------
    UBASIC.vg=UB_Basic_Editor_GetVGColor();
    UBASIC.bg=UB_Basic_Editor_GetBGColor();
    UBASIC.max_ram=(STM_BASIC_RAM_SIZE>>2);
    UBASIC.font_ptr=BasicEditorFont.font_name;
    
    // init vom uBasic und start vom pre-parser
    if(UB_BASIC.source==0) {
      // variabeln zuruecksetzen
      ubasic_init(basic_ram,0);
    }
    else {
      // variabeln nicht zuruecksetzen
      ubasic_init(inline_interpr,1);
    }
    if(ubasic_failed()) {
      // fehler vom pre-parser erkannt
      p_Basic_show_Fail();
      UB_BASIC.source=0;
    }
    else {
      // Mode auf "RUN_START"      
      UB_BASIC.mode=BASIC_MODE_RUN_START;

      // warte bis "RETURN" wieder verlassen
      do {
        key_anz=UB_USB_HID_HOST_GetKeyAnz();
      }while(key_anz>0);
      // screen loeschen mit bg color
      UB_Graphic2D_ClearBasicLCDBuffer(UBASIC.bg);
    }
  }
  else {
    // kein Basic-Program im Speicher
    UB_Basic_Editor_MoveCursor(MOVE_CUR_NEXTLINE);
    UB_Basic_Editor_WriteSystemString("BASIC RAM EMPTY");
    if(UB_BASIC.terminal==1) UB_Uart_SendString(COM1,"BASIC RAM EMPTY",CRLF);
    p_Basic_show_Ok();
  }
}


//--------------------------------------------------------------
// interne Funktion
// sendet ein ScreenCopy per UART als "BMP-File"
//--------------------------------------------------------------
void p_Basic_send_Screen(void)
{
  uint32_t n,adr;
  uint16_t x,y,color;
  uint8_t r,g,b;

  UB_Led_On(LED_RED);

  // BMP-Header senden
  for(n=0;n<54;n++) {
    UB_Uart_SendByte(COM1,BMP_HEADER[n]);
  }

  // den richigen Buffer zum senden raussuchen
  adr=LCD_CurrentFrameBuffer;

  // alle Farb-Daten senden
  for(x=0;x<LCD_MAXX;x++) {
    for(y=0;y<LCD_MAXY;y++) {
      #if USE_VGA_MODE==1
      n=(LCD_MAXX-x-1)*(LCD_MAXY*2)+(y*2);
      #else
      n=y*(LCD_MAXX*2)+(x*2);
      #endif
      color=*(volatile uint16_t*)(adr+n);
      r=((color&0xF800)>>8);  // 5bit rot
      g=((color&0x07E0)>>3);  // 6bit gruen
      b=((color&0x001F)<<3);  // 5bit blau
      UB_Uart_SendByte(COM1,b);
      UB_Uart_SendByte(COM1,g);
      UB_Uart_SendByte(COM1,r);
    }
  }

  UB_Led_Off(LED_RED);
}


