//--------------------------------------------------------------
// File     : stm32_ub_basic.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_BASIC_H
#define __STM32F4_UB_BASIC_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_basic_editor.h"
#include "stm32_ub_basic_interpreter.h"
#include "ubasic.h"
#include "stm32_ub_systick.h"
#include "stm32_ub_led.h"
#include "stm32_ub_usb_hid_host.h"
#include "stm32_ub_uart.h"
#include "stm32_ub_touch_stmpe811.h"
#include "stm32_ub_fatfs.h"


//--------------------------------------------------------------
// Versions Daten
//--------------------------------------------------------------
#define  VERSION_STRING   "1.00L"
#define  DATE_STRING      "20.12.2014"


//--------------------------------------------------------------
// Struktur von einem File
//--------------------------------------------------------------
typedef struct UB_File_t
{
  const uint8_t *table; // Tabelle mit den Daten
  uint32_t size;        // Anzahl der Bytes
}UB_File;


//--------------------------------------------------------------
// include aller Basic-Programme im Flash
// die Text-Files wurden per "File_Converter.exe"
// in C-Files gewandelt
//--------------------------------------------------------------
extern UB_File breakout;
extern UB_File calc;
extern UB_File clock;
extern UB_File gpio;
extern UB_File graphic;
extern UB_File hangman;
extern UB_File hello;
extern UB_File i2c;
extern UB_File key;
extern UB_File logic;
extern UB_File paint;
extern UB_File pong;
extern UB_File sprite;
extern UB_File string_bas;
extern UB_File uart;



//--------------------------------------------------------------
// Struktur von einem BASIC-Programm im Flash
//--------------------------------------------------------------
struct BASIC_ROM_t {
  uint8_t file_typ;   // typ (0=dir, 1=file)
  uint8_t akt_dir_nr; // direktory nr
  uint8_t new_dir_nr; // neu direktory nr (nur bei DIR)
  char *filename;     // name vom File oder Directory
  UB_File *fileptr;   // Pointer zum File
};



//--------------------------------------------------------------
// Speicherplatz fuer Basic-Programm (im externen SDRAM)
//--------------------------------------------------------------
#define  STM_BASIC_RAM_KB                        128    // RAM in kBytes
#define  STM_BASIC_RAM_SIZE   (STM_BASIC_RAM_KB*1024)   // in Bytes


//--------------------------------------------------------------
// Tastatur defines
//--------------------------------------------------------------
#define  KEY_REPEAT_MS       30  // Tastatur repeat (in ms)
#define  KEY_DELAY           15  // Delay nach 1tem Zeichen (in n*KEY_REPEAT_MS ms)


//--------------------------------------------------------------
// Display defines
//--------------------------------------------------------------
#define  CURSOR_BLINK_MS    250  // Blink Geschwindigkeit (in ms)
#define  LCD_REFRESH_MS      30  // Display Refresh Zeit (in ms)



//--------------------------------------------------------------
// sonstiges
//--------------------------------------------------------------
#define  UART_LOAD_TIMEOUT  5000000 // Timeout beim laden (start)
#define  UART_LOAD_TIMEOUT2  500000 // Timeout beim laden (weiter)



//--------------------------------------------------------------
// LCD Buffer 320x240 Pixel (im internen RAM)
// dieser wird zyklisch ans Display gesendet
//--------------------------------------------------------------
uint16_t basic_lcd_buffer[LCD_PIXEL];





//--------------------------------------------------------------
// enum vom aktuellen Basic-Mode
//--------------------------------------------------------------
typedef enum {
  BASIC_MODE_STARTUP =0, // Startup-Phase
  BASIC_MODE_EDITOR,     // Editor ist aktiv
  BASIC_MODE_LIST,       // Listing wird angezeigt
  BASIC_MODE_DIR,        // Directory wird angezeigt
  BASIC_MODE_RUN_START,  // Basic-Interpreter aktiv
  BASIC_MODE_RUN_STOP,   // Basic-Interpreter beendet
  BASIC_MODE_LOAD,       // laden per UART aktiv
}BASIC_MODE_t;


//--------------------------------------------------------------
// enum vom aktuellen Laufwerk
//--------------------------------------------------------------
typedef enum {
  BASIC_LW_UART =0,  // UART  (LOAD,SAVE)
  BASIC_LW_FLASH,    // FLASH (DIR,LOAD)
  BASIC_LW_SD,       // SD-Karte (DIR,LOAD,SAVE,REMOVE)
  BASIC_LW_USB,      // USB-Stick (DIR,LOAD,SAVE,REMOVE)
}BASIC_LW_t;


//--------------------------------------------------------------
// Struktur vom Basic
//--------------------------------------------------------------
typedef struct {
  BASIC_MODE_t mode;   // aktueller Mode
  uint8_t source;      // quelle vom Programm [RAM oder Inline]
  BASIC_LW_t lw;       // aktuelles Laufwerk
  uint8_t terminal;    // UART_Terminal 0=OFF; 1=ON
  uint8_t key_repeat;  // merker fuer Tastenrepeat
  uint32_t prg_size;   // groesse vom Programm im Basic-RAM
  uint32_t free_ram;   // freier speicher im Basic-RAM
  uint8_t akt_dir_nr;  // aktuelle direktory nr
}UB_BASIC_t;
UB_BASIC_t UB_BASIC;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_Basic_Init(void);
void UB_Basic_Start(void);
void UB_Basic_RefreshLCD(void);



//--------------------------------------------------------------
#endif // __STM32F4_UB_BASIC_H
