//--------------------------------------------------------------
// File     : stm32_ub_font.c
// Datum    : 20.12.2014
// Version  : 1.1b (STM32-Basic-Version)
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de 
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : STM32_UB_LCD_ILI9341
// Funktion : Text-LCD Funktionen
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_font.h"
#ifdef USE_VGA_SCREEN // Global define
  #include "stm32_ub_lcd_vga.h"
#else
  #include "stm32_ub_lcd_ili9341.h"
#endif
#include "stm32_ub_basic.h"



//--------------------------------------------------------------
// Zeichnet ein Ascii-Zeichen eines monospaced Fonts an x,y Position
// mit Vorder- und Hintergrundfarbe (Font = max 16 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
//--------------------------------------------------------------
/*
void UB_Font_DrawChar(uint16_t x, uint16_t y, uint8_t ascii, UB_Font *font, uint16_t vg, uint16_t bg)
{
  uint16_t xn,yn,start_maske,maske;
  const uint16_t *wert;
  

  ascii -= 32;
  wert=&font->table[ascii * font->height];

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    start_maske=0x80;
    if(font->width>8) start_maske=0x8000;
    
    for(yn = 0; yn < font->height; yn++) {
      maske=start_maske;
      // Cursor setzen
      UB_LCD_SetCursor2Draw(x,yn+y);
      for(xn = 0; xn < font->width; xn++) {
        if((wert[yn] & maske) == 0x00) {
          // Pixel in Hintergrundfarbe zeichnen
          UB_LCD_DrawPixel(bg);
        }
        else {
          // Pixel in Vordergrundfarbe zeichnen
          UB_LCD_DrawPixel(vg);
        }
        maske=(maske>>1);
      }
    }
  }
  else {
    start_maske=0x80;
    if(font->width>8) start_maske=0x8000;

    for(yn = 0; yn < font->height; yn++) {
      maske=start_maske;
      // Cursor setzen
      UB_LCD_SetCursor2Draw(x+font->height-yn,y);
      for(xn = 0; xn < font->width; xn++) {
        if((wert[yn] & maske) == 0x00) {
          // Pixel in Hintergrundfarbe zeichnen
          UB_LCD_DrawPixel(bg);
        }
        else {
          // Pixel in Vordergrundfarbe zeichnen
          UB_LCD_DrawPixel(vg);
        }
        maske=(maske>>1);
      }
    }
  }
}
*/


//--------------------------------------------------------------
// Zeichnet einen String eines monospaced Fonts an x,y Position
// mit Vorder- und Hintergrundfarbe (Font = max 16 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
//--------------------------------------------------------------
/*
void UB_Font_DrawString(uint16_t x, uint16_t y,char *ptr, UB_Font *font, uint16_t vg, uint16_t bg)
{
  uint16_t pos;

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    pos=x;
    while (*ptr != 0) {
      UB_Font_DrawChar(pos,y,*ptr,font,vg,bg);
      pos+=font->width;
      ptr++;
    }
  }
  else {
    pos=y;
    while (*ptr != 0) {
      UB_Font_DrawChar(x,pos,*ptr,font,vg,bg);
      pos+=font->width;
      ptr++;
    }
  }
}
*/

//--------------------------------------------------------------
// Zeichnet ein Ascii-Zeichen eines monospaced Fonts an x,y Position
// mit Vorder- und Hintergrundfarbe (Font = max 32 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
//--------------------------------------------------------------
/*
void UB_Font_DrawChar32(uint16_t x, uint16_t y, uint8_t ascii, UB_Font32 *font, uint16_t vg, uint16_t bg)
{
  uint16_t xn,yn;
  uint32_t start_maske,maske;
  const uint32_t *wert;


  ascii -= 32;
  wert=&font->table[ascii * font->height];

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    start_maske=0x80;
    if(font->width>8) start_maske=0x8000;
    if(font->width>16) start_maske=0x80000000;

    for(yn = 0; yn < font->height; yn++) {
      maske=start_maske;
      // Cursor setzen
      UB_LCD_SetCursor2Draw(x,yn+y);
      for(xn = 0; xn < font->width; xn++) {
        if((wert[yn] & maske) == 0x00) {
          // Pixel in Hintergrundfarbe zeichnen
          UB_LCD_DrawPixel(bg);
        }
        else {
          // Pixel in Vordergrundfarbe zeichnen
          UB_LCD_DrawPixel(vg);
        }
        maske=(maske>>1);
      }
    }
  }
  else {
    start_maske=0x80;
    if(font->width>8) start_maske=0x8000;
    if(font->width>16) start_maske=0x80000000;

    for(yn = 0; yn < font->height; yn++) {
      maske=start_maske;
      // Cursor setzen
      UB_LCD_SetCursor2Draw(x+font->height-yn,y);
      for(xn = 0; xn < font->width; xn++) {
        if((wert[yn] & maske) == 0x00) {
          // Pixel in Hintergrundfarbe zeichnen
          UB_LCD_DrawPixel(bg);
        }
        else {
          // Pixel in Vordergrundfarbe zeichnen
          UB_LCD_DrawPixel(vg);
        }
        maske=(maske>>1);
      }
    }
  }
}
*/

//--------------------------------------------------------------
// Zeichnet einen String eines monospaced Fonts an x,y Position
// mit Vorder- und Hintergrundfarbe (Font = max 32 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
//--------------------------------------------------------------
/*
void UB_Font_DrawString32(uint16_t x, uint16_t y,char *ptr, UB_Font32 *font, uint16_t vg, uint16_t bg)
{
  uint16_t pos;

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    pos=x;
    while (*ptr != 0) {
      UB_Font_DrawChar32(pos,y,*ptr,font,vg,bg);
      pos+=font->width;
      ptr++;
    }
  }
  else {
    pos=y;
    while (*ptr != 0) {
      UB_Font_DrawChar32(x,pos,*ptr,font,vg,bg);
      pos+=font->width;
      ptr++;
    }
  }
}
*/

//--------------------------------------------------------------
// Zeichnet ein Ascii-Zeichen eines proportional Fonts an x,y Position
// mit Vorder- und Hintergrundfarbe (Font = max 16 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
// return : breite des gezeichneten Zeichens
//--------------------------------------------------------------
/*
uint16_t UB_Font_DrawPChar(uint16_t x, uint16_t y, uint8_t ascii, UB_pFont *font, uint16_t vg, uint16_t bg)
{
  uint16_t xn,yn,start_maske,maske,width;
  const uint16_t *wert;

  // check auf Grenze
  if(ascii<font->first_char) return 0;
  if(ascii>font->last_char) return 0;

  ascii -= font->first_char;
  wert=&font->table[ascii * (font->height+1)];
  width=wert[0];

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    start_maske=0x01;
    start_maske=start_maske<<(width-1);

    for(yn = 0; yn < font->height; yn++) {
      maske=start_maske;
      // Cursor setzen
      UB_LCD_SetCursor2Draw(x,yn+y);
      for(xn = 0; xn < width; xn++) {
        if((wert[yn+1] & maske) == 0x00) {
          // Pixel in Hintergrundfarbe zeichnen
          UB_LCD_DrawPixel(bg);
        }
        else {
          // Pixel in Vordergrundfarbe zeichnen
          UB_LCD_DrawPixel(vg);
        }
        maske=(maske>>1);
      }
    }
  }
  else {
    start_maske=0x01;
    start_maske=start_maske<<(width-1);

    for(yn = 0; yn < font->height; yn++) {
      maske=start_maske;
      // Cursor setzen
      UB_LCD_SetCursor2Draw(x+font->height-yn,y);
      for(xn = 0; xn < width; xn++) {
        if((wert[yn+1] & maske) == 0x00) {
          // Pixel in Hintergrundfarbe zeichnen
          UB_LCD_DrawPixel(bg);
        }
        else {
          // Pixel in Vordergrundfarbe zeichnen
          UB_LCD_DrawPixel(vg);
        }
        maske=(maske>>1);
      }
    }
  }

  return(width);
}
*/

//--------------------------------------------------------------
// Zeichnet einen String eines proportional Fonts an x,y Position
// mit Vorder- und Hintergrundfarbe (Font = max 16 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
//--------------------------------------------------------------
/*
void UB_Font_DrawPString(uint16_t x, uint16_t y,char *ptr, UB_pFont *font, uint16_t vg, uint16_t bg)
{
  uint16_t pos,width;

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    pos=x;
    while (*ptr != 0) {
      width=UB_Font_DrawPChar(pos,y,*ptr,font,vg,bg);
      pos+=width;
      ptr++;
    }
  }
  else {
    pos=y;
    while (*ptr != 0) {
      width=UB_Font_DrawPChar(x,pos,*ptr,font,vg,bg);
      pos+=width;
      ptr++;
    }
  }
}
*/

//--------------------------------------------------------------
// Zeichnet ein Ascii-Zeichen eines proportional Fonts an x,y Position
// mit Vorder- und Hintergrundfarbe (Font = max 32 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
// return : breite des gezeichneten Zeichens
//--------------------------------------------------------------
/*
uint16_t UB_Font_DrawPChar32(uint16_t x, uint16_t y, uint8_t ascii, UB_pFont32 *font, uint16_t vg, uint16_t bg)
{
  uint16_t xn,yn,width;
  uint32_t start_maske,maske;
  const uint32_t *wert;

  // check auf Grenze
  if(ascii<font->first_char) return 0;
  if(ascii>font->last_char) return 0;

  ascii -= font->first_char;
  wert=&font->table[ascii * (font->height+1)];
  width=wert[0];

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    start_maske=0x01;
    start_maske=start_maske<<(width-1);

    for(yn = 0; yn < font->height; yn++) {
      maske=start_maske;
      // Cursor setzen
      UB_LCD_SetCursor2Draw(x,yn+y);
      for(xn = 0; xn < width; xn++) {
        if((wert[yn+1] & maske) == 0x00) {
          // Pixel in Hintergrundfarbe zeichnen
          UB_LCD_DrawPixel(bg);
        }
        else {
          // Pixel in Vordergrundfarbe zeichnen
          UB_LCD_DrawPixel(vg);
        }
        maske=(maske>>1);
      }
    }
  }
  else {
    start_maske=0x01;
    start_maske=start_maske<<(width-1);

    for(yn = 0; yn < font->height; yn++) {
      maske=start_maske;
      // Cursor setzen
      UB_LCD_SetCursor2Draw(x+font->height-yn,y);
      for(xn = 0; xn < width; xn++) {
        if((wert[yn+1] & maske) == 0x00) {
          // Pixel in Hintergrundfarbe zeichnen
          UB_LCD_DrawPixel(bg);
        }
        else {
          // Pixel in Vordergrundfarbe zeichnen
          UB_LCD_DrawPixel(vg);
        }
        maske=(maske>>1);
      }
    }
  }

  return(width);
}
*/

//--------------------------------------------------------------
// Zeichnet einen String eines proportional Fonts an x,y Position
// mit Vorder- und Hintergrundfarbe (Font = max 32 Pixel breite)
// -> Font muss mit &-Operator uebergeben werden
//--------------------------------------------------------------
/*
void UB_Font_DrawPString32(uint16_t x, uint16_t y,char *ptr, UB_pFont32 *font, uint16_t vg, uint16_t bg)
{
  uint16_t pos,width;

  if(LCD_DISPLAY_MODE==PORTRAIT) {
    pos=x;
    while (*ptr != 0) {
      width=UB_Font_DrawPChar32(pos,y,*ptr,font,vg,bg);
      pos+=width;
      ptr++;
    }
  }
  else {
    pos=y;
    while (*ptr != 0) {
      width=UB_Font_DrawPChar32(x,pos,*ptr,font,vg,bg);
      pos+=width;
      ptr++;
    }
  }
}
*/

//--------------------------------------------------------------
// zeichnet ein Zeichen in den Basic LCD-Buffer (nur Landscape)
// x     = spalte [0...319] 0=links
// y     = zeile  [0...219] 0=oben
// ascii = ascii-nr vom Zeichen
// font  = Schriftart 
// vg    = Textfarbe        [16bit]
// bg    = Hintergrundfarbe [16bit] 
//
// Adressen vom LCD-Buffer : (LCD)
//   linke untere Ecke : adr=0
//   linke obere Ecke  : adr=239
//   pro Pixel in X-Richtung : adr+=240 (nach rechts)
//   pro Pixel in Y-Richtung : adr-=1   (nach unten)
//
// Adressen vom LCD-Buffer : (VGA)
//   linke obere Ecke : adr=0
//   rechte obere Ecke  : adr=319
//   pro Pixel in X-Richtung : adr+=1   (nach rechts)
//   pro Pixel in Y-Richtung : adr+=320 (nach unten)
//--------------------------------------------------------------
void UB_Font_BasicChar(uint16_t x, uint16_t y, uint8_t ascii, UB_Font *font, uint16_t vg, uint16_t bg)
{
  uint16_t xn,yn,start_maske,maske,value;
  const uint16_t *wert;
  uint32_t ram_adr=0;
  uint32_t offset;

  // check auf Grenzwerte
  if((x+font->width)>LCD_MAXY) return;
  if((y+font->height)>LCD_MAXX) return;

  ascii -= 32;
  wert=&font->table[ascii * font->height];

  start_maske=0x80;
  if(font->width>8) start_maske=0x8000;

#if USE_VGA_MODE==1
  // offset fuer Landscape-Mode ausrechnen
  offset=(LCD_MAXY*y)+x;

  // komplettes Zeichen anzeigen
  for(yn = 0; yn < font->height; yn++) {
    maske=start_maske;
    value=wert[yn];
    // Adresse ausrechnen
    ram_adr=(uint32_t)(offset+(yn*LCD_MAXY));
    for(xn = 0; xn < font->width; xn++) {
      if((value & maske) == 0x00) {
        // Pixel in Hintergrundfarbe zeichnen
        basic_lcd_buffer[ram_adr]=bg;
      }
      else {
        // Pixel in Vordergrundfarbe zeichnen
        basic_lcd_buffer[ram_adr]=vg;
      }
      maske=(maske>>1);
      ram_adr++;
    }
  }
#else

  // offset fuer Landscape-Mode ausrechnen
  offset=(LCD_MAXX*x)+(LCD_MAXX-1-y);

  // komplettes Zeichen anzeigen
  for(yn = 0; yn < font->height; yn++) {
    maske=start_maske;
    value=wert[yn];
    // Adresse ausrechnen
    ram_adr=(uint32_t)(offset-yn);
    for(xn = 0; xn < font->width; xn++) {
      if((value & maske) == 0x00) {
        // Pixel in Hintergrundfarbe zeichnen
        basic_lcd_buffer[ram_adr]=bg;
      }
      else {
        // Pixel in Vordergrundfarbe zeichnen
        basic_lcd_buffer[ram_adr]=vg;
      }
      maske=(maske>>1);
      ram_adr+=LCD_MAXX;
    }
  }
#endif
}


//--------------------------------------------------------------
// zeichnet einen String in den Basic LCD-Buffer (nur Landscape)
// x     = spalte [0...319] 0=links
// y     = zeile  [0...219] 0=oben
// ptr   = String
// font  = Schriftart
// vg    = Textfarbe        [16bit]
// bg    = Hintergrundfarbe [16bit]
//--------------------------------------------------------------
void UB_Font_BasicString(uint16_t x, uint16_t y, char *ptr, UB_Font *font, uint16_t vg, uint16_t bg)
{
  uint16_t posx;

   posx=x;
   while (*ptr != 0) {
     UB_Font_BasicChar(posx,y,*ptr,font,vg,bg);
     posx+=font->width;     
     ptr++;
   }
}
