//--------------------------------------------------------------
// File     : stm32_ub_systick.c
// Datum    : 18.01.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : keine
// Funktion : Pausen- Timer- und Counter-Funktionen
//            Zeiten im [us,ms,s] Raster
//--------------------------------------------------------------

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_systick.h"



#if ((SYSTICK_RESOLUTION!=1) && (SYSTICK_RESOLUTION!=1000))
  #error print WRONG SYSTICK RESOLUTION !
#endif


//--------------------------------------------------------------
// Globale Pausen-Variabeln
//--------------------------------------------------------------
static volatile uint32_t Systick_Delay;  // Globaler Pausenzaehler





//--------------------------------------------------------------
// Init vom System-Counter
// entweder im 1us-Takt oder 1ms-Takt
//--------------------------------------------------------------
void UB_Systick_Init(void) {
  RCC_ClocksTypeDef RCC_Clocks;

  // alle Variabeln zurcksetzen
  Systick_Delay=0;
  keyboard_timer=0;
  cursor_blink_timer=0;
  lcd_refresh_timer=LCD_REFRESH_MS;
  basic_timer=0;
  basic_tic=0;
  akt_usb_status=USB_HID_DEV_DETACHED;


  #if SYSTICK_RESOLUTION==1
    // Timer auf 1us einstellen
    RCC_GetClocksFreq(&RCC_Clocks);
    SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000000);
  #else
    // Timer auf 1ms einstellen
    RCC_GetClocksFreq(&RCC_Clocks);
    SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000);
  #endif
}


#if SYSTICK_RESOLUTION==1
//--------------------------------------------------------------
// Pausenfunktion (in us)
// die CPU wartet bis die Zeit abgelaufen ist
//--------------------------------------------------------------
void UB_Systick_Pause_us(volatile uint32_t pause)
{

  Systick_Delay = pause;

  while(Systick_Delay != 0);
}
#endif


//--------------------------------------------------------------
// Pausenfunktion (in ms)
// die CPU wartet bis die Zeit abgelaufen ist
//--------------------------------------------------------------
void UB_Systick_Pause_ms(volatile uint32_t pause)
{
  #if SYSTICK_RESOLUTION==1
    uint32_t ms;

    for(ms=0;ms<pause;ms++) {
      UB_Systick_Pause_us(1000);
    }
  #else
    Systick_Delay = pause;

    while(Systick_Delay != 0);
  #endif
}


//--------------------------------------------------------------
// Pausenfunktion (in s)
// die CPU wartet bis die Zeit abgelaufen ist
//--------------------------------------------------------------
void UB_Systick_Pause_s(volatile uint32_t pause)
{
  uint32_t s;

  for(s=0;s<pause;s++) {
    UB_Systick_Pause_ms(1000);
  }
}


//--------------------------------------------------------------
// fr Basic
// lschen vom Timer
//--------------------------------------------------------------
void UB_Systick_BasicResetTimer(void)
{
  basic_timer=0;
}

//--------------------------------------------------------------
// fr Basic
// auslesen vom Timer
//--------------------------------------------------------------
uint32_t UB_Systick_BasicReadTimer(void)
{
  return(basic_timer);
}

//--------------------------------------------------------------
// fr Basic
// lschen vom Systick
//--------------------------------------------------------------
void UB_Systick_BasicResetTIC(void)
{
  basic_tic=0;
}

//--------------------------------------------------------------
// fr Basic
// auslesen vom Systick
//--------------------------------------------------------------
uint32_t UB_Systick_BasicReadTIC(void)
{
  return(basic_tic);
}

//--------------------------------------------------------------
// Systic-Interrupt
//--------------------------------------------------------------
void SysTick_Handler(void)
{
  // Tick fr Pause
  if(Systick_Delay != 0x00) Systick_Delay--;

  // USB bearbeiten
  akt_usb_status=UB_USB_HID_HOST_Do();

  // Keyboard Timer
  if(keyboard_timer!=0) keyboard_timer--;

  // Blink Timer fuer Cursor
  if(cursor_blink_timer!=0) cursor_blink_timer--;

  // Basic-Timer
  basic_timer++;

  // Basic-TIC
  basic_tic++;

  // LCD Timer
  if(UB_BASIC.mode!=BASIC_MODE_STARTUP) {
    lcd_refresh_timer--;
    if(lcd_refresh_timer==0) {
      lcd_refresh_timer=LCD_REFRESH_MS;
      // Refresh vom LCD
      UB_Basic_RefreshLCD();
    }
  }
}


