//--------------------------------------------------------------
// File     : stm32_ub_basic_editor.c
// Datum    : 20.12.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : STM32_UB_FONT, STM32_UB_GRAPHIC_2D
// Funktion : Basic Editor
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_basic_editor.h"



//--------------------------------------------------------------
// Globale Struktur
//--------------------------------------------------------------
BASIC_EDITOR_t BASIC_EDITOR;



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void p_Editor_setDefaultColorTable(void);
uint8_t p_Editor_getAktColor(void);
void p_Editor_clearScreen(void);
void p_Editor_moveScreenUp(void);
void p_Editor_moveRowLeft(uint8_t offset);
void p_Editor_moveRowRight(void);
void p_Editor_showCursor(void);
void p_Editor_updateScreenRAM(uint8_t row);



//--------------------------------------------------------------
// init vom Basic-Editor
//--------------------------------------------------------------
void UB_Basic_Editor_Init(void)
{
  // Editor-Struktur initialisiern
  BasicEditorFont.font_nr=0;
  BasicEditorFont.font_name=&C64_8x8;
  BasicEditorFont.char_anzx=LCD_MAXY/BasicEditorFont.font_name->width;
  BasicEditorFont.char_anzy=LCD_MAXX/BasicEditorFont.font_name->height;
  BASIC_EDITOR.color.vg_col=EDITOR_DEF_VG_COL;
  BASIC_EDITOR.color.bg_col=EDITOR_DEF_BG_COL;
  BASIC_EDITOR.color.sys_col=EDITOR_DEF_SYS_COL;  
  BASIC_EDITOR.cursor.xpos=0;
  BASIC_EDITOR.cursor.ypos=0;
  BASIC_EDITOR.cursor.visible=0;

  // Farbtabellen auf default
  p_Editor_setDefaultColorTable();
}


//--------------------------------------------------------------
// reset vom Basic-Editor
//--------------------------------------------------------------
void UB_Basic_Editor_Reset(void)
{
  // reset aller Variabeln
  UB_Basic_Editor_Init();

  // loeschen vom Display
  UB_Basic_Editor_ClearSreen();
}

//--------------------------------------------------------------
// loescht den Bildschirm
// und setzt Cursor auf 0,0 (links,oben)
//--------------------------------------------------------------
void UB_Basic_Editor_ClearSreen(void)
{
  // screen loeschen
  p_Editor_clearScreen();

  // Cursor auf 0,0 setzen
  UB_Basic_Editor_SetCursor(0,0);

  // update vom Screen
  UB_Basic_Editor_UpdateSreen();
}


//--------------------------------------------------------------
// zeigt den kompletten Screen-Inhalt an
//--------------------------------------------------------------
void UB_Basic_Editor_UpdateSreen(void)
{
  // Screen komplett kopieren
  p_Editor_updateScreenRAM(99);
}


//--------------------------------------------------------------
// schreibt ein Zeichen an aktuelle Cursor-Position
// mit aktueller VG+BG Color (Ueberschreib-Mode)
// (verschiebt danach den Cursor)
// ascii : Ascii-Nr vom Zeichen [0...255]
// Durchlaufzeit ca. 20us (wenn Cursor gerade AUS)
// Durchlaufzeit ca. 40us (wenn Cursor gerade EIN)
//--------------------------------------------------------------
void UB_Basic_Editor_WriteChar(uint8_t ascii_value)
{
  uint32_t x,y;
  uint16_t vg,bg;
  uint16_t xp,yp;

  // check auf Maxwerte
  // bei falschem Zeichen ein "?" anzeigen
  if(ascii_value<EDITOR_FIRST_CHAR) ascii_value='?';
  if(ascii_value>EDITOR_LAST_CHAR) ascii_value='?';

  // Cursor-Position ermitteln
  x=BASIC_EDITOR.cursor.xpos;
  y=BASIC_EDITOR.cursor.ypos;

  // Zeichen in Text-RAM schreiben
  EDITOR_TEXT_RAM[x][y]=ascii_value;

  // Farbe ins Color-RAM schreiben
  EDITOR_COLOR_RAM[x][y]=p_Editor_getAktColor();

  // Zeichen ins Screen-RAM schreiben
  vg=EDITOR_COLOR_TABLE_VG[BASIC_EDITOR.color.vg_col];
  bg=EDITOR_COLOR_TABLE_BG[BASIC_EDITOR.color.bg_col];
  xp=x*BasicEditorFont.font_name->width;
  yp=y*BasicEditorFont.font_name->height;
  UB_Font_BasicChar(xp,yp,ascii_value,BasicEditorFont.font_name,vg,bg);

  // Cursor verschieben
  UB_Basic_Editor_MoveCursor(MOVE_CUR_RIGHT);
}


//--------------------------------------------------------------
// schreibt einen String an aktuelle Cursor-Position
// mit aktueller VG+BG Color (Ueberschreib-Mode)
// (verschiebt danach den Cursor)
//--------------------------------------------------------------
void UB_Basic_Editor_WriteString(char *ptr)
{
  while (*ptr != 0) {
    UB_Basic_Editor_WriteChar(*ptr);
    ptr++;
  }
}


//--------------------------------------------------------------
// schreibt einen String an aktuelle Cursor-Position
// mit aktueller SYS+BG Color (Ueberschreib-Mode)
// (verschiebt danach den Cursor)
//--------------------------------------------------------------
void UB_Basic_Editor_WriteSystemString(char *ptr)
{
  uint8_t vg_col;

  // Text Farbe sichern
  vg_col=BASIC_EDITOR.color.vg_col;
  // System Farbe benutzen
  BASIC_EDITOR.color.vg_col=BASIC_EDITOR.color.sys_col;

  while (*ptr != 0) {
    UB_Basic_Editor_WriteChar(*ptr);
    ptr++;
  }  

  // Text Farbe wieder herstellen
  BASIC_EDITOR.color.vg_col=vg_col;
}


//--------------------------------------------------------------
// schreibt ein Zeichen an aktuelle Cursor-Position
// mit aktueller VG+BG Color (Einfuege-Mode)
// (verschiebt die Zeile und den Cursor)
// ascii : Ascii-Nr vom Zeichen [0...255]
//--------------------------------------------------------------
void UB_Basic_Editor_InsertChar(uint8_t ascii_value)
{
  uint32_t x;

  // check auf Maxwerte
  if(ascii_value<EDITOR_FIRST_CHAR) return;
  if(ascii_value>EDITOR_LAST_CHAR) return;

  // Cursor-Position ermitteln
  x=BASIC_EDITOR.cursor.xpos;

  // nur wenn nicht letztes Zeichen
  if((x<BasicEditorFont.char_anzx-1)) {
    // aktuelle Zeile nach rechts schieben
    p_Editor_moveRowRight();
  }
  // zeichen schreiben
  UB_Basic_Editor_WriteChar(ascii_value);
}


//--------------------------------------------------------------
// loescht das Zeichen links von aktueller Cursor-Position
// (verschiebt die Zeile und den Cursor)
//--------------------------------------------------------------
void UB_Basic_Editor_BackspaceChar(void)
{
  // nur wenn nicht erstes zeichen
  if(BASIC_EDITOR.cursor.xpos>0) {
    // aktuelle Zeile nach links schieben
    p_Editor_moveRowLeft(0);
  }
  // cursor ein zeichen nach links bewegen
  UB_Basic_Editor_MoveCursor(MOVE_CUR_LEFT);
}


//--------------------------------------------------------------
// loescht das Zeichen unter der aktueller Cursor-Position
// (verschiebt die Zeile)
//--------------------------------------------------------------
void UB_Basic_Editor_DeleteChar(void)
{
  // aktuelle Zeile nach links schieben
  p_Editor_moveRowLeft(1);
}


//--------------------------------------------------------------
// bewegt den Cursor auf dem Screen
// direction :
//   MOVE_CUR_UP       = ein Zeichen hoch
//   MOVE_CUR_DOWN     = ein Zeichen runter
//   MOVE_CUR_LEFT     = ein Zeichen nach links
//   MOVE_CUR_RIGHT    = ein Zeichen nach Rechts
//   MOVE_CUR_X0       = auf Spalte=0 in aktueller Zeile
//   MOVE_CUR_X0Y0     = auf X=0, Y=0
//   MOVE_CUR_NEXTLINE = auf X=0, Y++
//   MOVE_CUR_LASTCHAR = auf letztes Zeichen in aktueller Zeile
//--------------------------------------------------------------
void UB_Basic_Editor_MoveCursor(MOVE_CURSOR_t direction)
{
  uint32_t x,y;

  // check ob cursor an alter stelle sichtbar war
  if(BASIC_EDITOR.cursor.visible==1) {
    // cursor ausblenden
    UB_Basic_Editor_ToggleCursor();
  }

  if(direction==MOVE_CUR_UP) {
    if(BASIC_EDITOR.cursor.ypos>0) BASIC_EDITOR.cursor.ypos--;
  }
  else if(direction==MOVE_CUR_DOWN) {
    BASIC_EDITOR.cursor.ypos++;
    if(BASIC_EDITOR.cursor.ypos>=BasicEditorFont.char_anzy) {
      BASIC_EDITOR.cursor.ypos=(BasicEditorFont.char_anzy-1);
      // kompletter editor bereich
      // um eine zeile nach oben schieben
      p_Editor_moveScreenUp();      
    }
  }
  else if(direction==MOVE_CUR_LEFT) {
    if(BASIC_EDITOR.cursor.xpos>0) {
      BASIC_EDITOR.cursor.xpos--;
    }
    else {
      if(BASIC_EDITOR.cursor.ypos>0) {
        BASIC_EDITOR.cursor.xpos=(BasicEditorFont.char_anzx-1);
        BASIC_EDITOR.cursor.ypos--;
      }
    }
  }
  else if(direction==MOVE_CUR_RIGHT) {
    BASIC_EDITOR.cursor.xpos++;
    if(BASIC_EDITOR.cursor.xpos>=BasicEditorFont.char_anzx) {
      BASIC_EDITOR.cursor.xpos=0;
      BASIC_EDITOR.cursor.ypos++;
      if(BASIC_EDITOR.cursor.ypos>=BasicEditorFont.char_anzy) {
        BASIC_EDITOR.cursor.ypos=(BasicEditorFont.char_anzy-1);
        // kompletter editor bereich
        // um eine zeile nach oben schieben
        p_Editor_moveScreenUp();        
      }
    }
  }
  else if(direction==MOVE_CUR_X0) {
    BASIC_EDITOR.cursor.xpos=0;
  }
  else if(direction==MOVE_CUR_X0Y0) {
    BASIC_EDITOR.cursor.xpos=0;
    BASIC_EDITOR.cursor.ypos=0;
  }
  else if(direction==MOVE_CUR_NEXTLINE) {
    BASIC_EDITOR.cursor.xpos=0;
    BASIC_EDITOR.cursor.ypos++;
    if(BASIC_EDITOR.cursor.ypos>=BasicEditorFont.char_anzy) {
      BASIC_EDITOR.cursor.ypos=(BasicEditorFont.char_anzy-1);
      // kompletter editor bereich
      // um eine zeile nach oben schieben
      p_Editor_moveScreenUp();
    }
  }
  else if(direction==MOVE_CUR_LASTCHAR) {
    y=BASIC_EDITOR.cursor.ypos;
    BASIC_EDITOR.cursor.xpos=BasicEditorFont.char_anzx-1;
    for(x=0;x<BasicEditorFont.char_anzx;x++) {
      if(EDITOR_TEXT_RAM[x][y]!=EDITOR_SPACE_CHAR) {
        BASIC_EDITOR.cursor.xpos=x;
      }
    }
  }
}


//--------------------------------------------------------------
// setzt den Cursor auf die Position X/Y
// x : [0...BASIC_EDITOR_ANZX-1] 0=links
// y : [0...BASIC_EDITOR_ANZY-1] 0=oben
//--------------------------------------------------------------
void UB_Basic_Editor_SetCursor(uint32_t x, uint32_t y)
{

  // check ob cursor an alter stelle sichtbar war
  if(BASIC_EDITOR.cursor.visible==1) {
    // cursor ausblenden
    UB_Basic_Editor_ToggleCursor();
  }

  BASIC_EDITOR.cursor.xpos=x;
  if(BASIC_EDITOR.cursor.xpos>=BasicEditorFont.char_anzx) {
    BASIC_EDITOR.cursor.xpos=(BasicEditorFont.char_anzx-1);
  }

  BASIC_EDITOR.cursor.ypos=y;
  if(BASIC_EDITOR.cursor.ypos>=BasicEditorFont.char_anzy) {
    BASIC_EDITOR.cursor.ypos=(BasicEditorFont.char_anzy-1);
  }
}


//--------------------------------------------------------------
// toggelt den Cursor zwischen
// normal und invers
//--------------------------------------------------------------
void UB_Basic_Editor_ToggleCursor(void)
{
  BASIC_EDITOR.cursor.visible=1-BASIC_EDITOR.cursor.visible;
  p_Editor_showCursor();
}


//--------------------------------------------------------------
// stellt die Textfarbe ein
// color : Farbe [0...15]
//--------------------------------------------------------------
void UB_Basic_Editor_SetVGColor(uint8_t color)
{
  if(color>15) color=15;

  BASIC_EDITOR.color.vg_col=color;
}


//--------------------------------------------------------------
// liefert die aktuelle Textfarbe als 16bit Wert zurueck
// color : Farbe 0...65535
//--------------------------------------------------------------
uint16_t UB_Basic_Editor_GetVGColor(void)
{
  return(EDITOR_COLOR_TABLE_VG[BASIC_EDITOR.color.vg_col]);
}

//--------------------------------------------------------------
// stellt die Hintergrundfarbe ein
// color : Farbe [0...15]
//--------------------------------------------------------------
void UB_Basic_Editor_SetBGColor(uint8_t color)
{
  if(color>15) color=15;

  BASIC_EDITOR.color.bg_col=color;
}


//--------------------------------------------------------------
// liefert die aktuelle Hintergrundfarbe als 16bit Wert zurueck
// color : Farbe 0...65535
//--------------------------------------------------------------
uint16_t UB_Basic_Editor_GetBGColor(void)
{
  return(EDITOR_COLOR_TABLE_BG[BASIC_EDITOR.color.bg_col]);
}


//--------------------------------------------------------------
// stellt die Systemfarbe ein
// color : Farbe [0...15]
//--------------------------------------------------------------
void UB_Basic_Editor_SetSYSColor(uint8_t color)
{
  if(color>15) color=15;

  BASIC_EDITOR.color.sys_col=color;
}


//--------------------------------------------------------------
// wechselt den Zeichensatz
// (schaltet zum naechsten Zeichensatz weiter)
//--------------------------------------------------------------
void UB_Basic_Editor_ChangeFont(void)
{
  // cursor auf 0,0
  UB_Basic_Editor_SetCursor(0,0);

  // Font wechseln
  BasicEditorFont.font_nr++;
  if(BasicEditorFont.font_nr>4) BasicEditorFont.font_nr=0; // font 0-4


  if(BasicEditorFont.font_nr==0) {
    BasicEditorFont.font_name=&C64_8x8; // 8x8 Pixel
  }
  else if(BasicEditorFont.font_nr==1) {
    BasicEditorFont.font_name=&Arial_8x13; // 8x13 Pixel
  }
  else if(BasicEditorFont.font_nr==2) {
    BasicEditorFont.font_name=&Arial_8x13b; // 8x13 Pixel
  }
  else if(BasicEditorFont.font_nr==3) {
    BasicEditorFont.font_name=&Arial_10x15; // 10x15 Pixel
  }
  else if(BasicEditorFont.font_nr==4) {
    BasicEditorFont.font_name=&Arial_10x15b; // 10x15 Pixel
  }

  // anzahl der Zeichen ausrechnen
  BasicEditorFont.char_anzx=LCD_MAXY/BasicEditorFont.font_name->width;
  BasicEditorFont.char_anzy=LCD_MAXX/BasicEditorFont.font_name->height;

  // screen loeschen
  p_Editor_clearScreen();
}


//--------------------------------------------------------------
// kopiert die aktuelle Zeile vom Cursor als String
// ret_wert = laenge vom kopierten string (0=kein Inhalt)
// Leerzeichen (Space) vorne und hinten werden nicht mitkopiert
//--------------------------------------------------------------
uint8_t UB_Basic_Editor_CopyLine(char *ptr)
{
  uint8_t ret_wert=0;
  uint8_t start=0, stop=0;
  uint32_t x,y,laenge;

  // aktuelle Zeilen-Nummer
  y=BASIC_EDITOR.cursor.ypos;

  // start und ende vom string suchen
  for(x=0;x<BasicEditorFont.char_anzx;x++) {
    if(EDITOR_TEXT_RAM[x][y]!=EDITOR_SPACE_CHAR) {
      if(start==0) start=x+1;
      if(stop<(x+1)) stop=x+1;
    }
  }

  if(start>0) {
    // alle zeichen vom String kopieren
    laenge=stop-start+1;
    for(x=0;x<laenge;x++) {
      ptr[x]=EDITOR_TEXT_RAM[x+start-1][y];
    }
    ptr[x]=0x00; // String Endekennung
    ret_wert=laenge;
  }
  else {
    // zeile ist leer
    ptr[0]=0x00;
    ret_wert=0;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// zeigt zu Debug-Zwecken einen String an
//--------------------------------------------------------------
void UB_Basic_Editor_DebugString(char *ptr)
{
  uint32_t x,y;

  x=BASIC_EDITOR.cursor.xpos;
  y=BASIC_EDITOR.cursor.ypos;

  UB_Basic_Editor_SetCursor(0,29);
  UB_Basic_Editor_WriteString(ptr);

  UB_Basic_Editor_SetCursor(x,y);
}


//--------------------------------------------------------------
// interne Funktion
// Farbtabellen auf default
//--------------------------------------------------------------
void p_Editor_setDefaultColorTable(void)
{
  // Text-Farben auf standard
  EDITOR_COLOR_TABLE_VG[0]=EDITOR_COL_0;
  EDITOR_COLOR_TABLE_VG[1]=EDITOR_COL_1;
  EDITOR_COLOR_TABLE_VG[2]=EDITOR_COL_2;
  EDITOR_COLOR_TABLE_VG[3]=EDITOR_COL_3;
  EDITOR_COLOR_TABLE_VG[4]=EDITOR_COL_4;
  EDITOR_COLOR_TABLE_VG[5]=EDITOR_COL_5;
  EDITOR_COLOR_TABLE_VG[6]=EDITOR_COL_6;
  EDITOR_COLOR_TABLE_VG[7]=EDITOR_COL_7;
  EDITOR_COLOR_TABLE_VG[8]=EDITOR_COL_8;
  EDITOR_COLOR_TABLE_VG[9]=EDITOR_COL_9;
  EDITOR_COLOR_TABLE_VG[10]=EDITOR_COL_10;
  EDITOR_COLOR_TABLE_VG[11]=EDITOR_COL_11;
  EDITOR_COLOR_TABLE_VG[12]=EDITOR_COL_12;
  EDITOR_COLOR_TABLE_VG[13]=EDITOR_COL_13;
  EDITOR_COLOR_TABLE_VG[14]=EDITOR_COL_14;
  EDITOR_COLOR_TABLE_VG[15]=EDITOR_COL_15;

  // Hintergrund-Farben auf standard
  EDITOR_COLOR_TABLE_BG[0]=EDITOR_COL_0;
  EDITOR_COLOR_TABLE_BG[1]=EDITOR_COL_1;
  EDITOR_COLOR_TABLE_BG[2]=EDITOR_COL_2;
  EDITOR_COLOR_TABLE_BG[3]=EDITOR_COL_3;
  EDITOR_COLOR_TABLE_BG[4]=EDITOR_COL_4;
  EDITOR_COLOR_TABLE_BG[5]=EDITOR_COL_5;
  EDITOR_COLOR_TABLE_BG[6]=EDITOR_COL_6;
  EDITOR_COLOR_TABLE_BG[7]=EDITOR_COL_7;
  EDITOR_COLOR_TABLE_BG[8]=EDITOR_COL_8;
  EDITOR_COLOR_TABLE_BG[9]=EDITOR_COL_9;
  EDITOR_COLOR_TABLE_BG[10]=EDITOR_COL_10;
  EDITOR_COLOR_TABLE_BG[11]=EDITOR_COL_11;
  EDITOR_COLOR_TABLE_BG[12]=EDITOR_COL_12;
  EDITOR_COLOR_TABLE_BG[13]=EDITOR_COL_13;
  EDITOR_COLOR_TABLE_BG[14]=EDITOR_COL_14;
  EDITOR_COLOR_TABLE_BG[15]=EDITOR_COL_15;
}


//--------------------------------------------------------------
// interne Funktion
// uebergibt aktuelle Text- und Hintergrundfarbe
// als 8bit Wert fuer das Color-RAM
// ret_wert : [0...255]
//  Bit0...3 = Textfarbe [0...15]
//  Bit5...7 = Hintergrundfarbe [0...15]
//--------------------------------------------------------------
uint8_t p_Editor_getAktColor(void)
{
  uint8_t ret_wert=0;

  ret_wert=(BASIC_EDITOR.color.bg_col<<4);
  ret_wert|=BASIC_EDITOR.color.vg_col;

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// Screen loeschen
//  Text-Ram mit Space fuellen
//  Color-Ram mit aktuellen farbwerten fuellen
//  LCD-Buffer mit Hintergrundfarbe loeschen
//--------------------------------------------------------------
void p_Editor_clearScreen(void)
{
  uint32_t x,y;
  uint16_t col;

  // kompletter Screen loeschen
  for(y=0;y<BasicEditorFont.char_anzy;y++) {
    for(x=0;x<BasicEditorFont.char_anzx;x++) {
      // Text-RAM loeschen (mit Space)
      EDITOR_TEXT_RAM[x][y]=EDITOR_SPACE_CHAR;
      // Color-RAM loeschen (mit aktueller VG und BG Farbe)
      EDITOR_COLOR_RAM[x][y]=p_Editor_getAktColor();
    }
  }

  // RAM loeschen per DMA
  col=EDITOR_COLOR_TABLE_BG[BASIC_EDITOR.color.bg_col];
  UB_Graphic2D_ClearBasicLCDBuffer(col);
}


//--------------------------------------------------------------
// interne Funktion
// verschiebt den kompletten Editor-Bereich
// um eine Zeile nach oben
// 1. die erste Zeile geht damit verloren
// 2. die letzte Zeile wird mit Space beschrieben
//--------------------------------------------------------------
void p_Editor_moveScreenUp(void)
{
  uint32_t x,y;

  // Text- und Color-RAM gleichzeitig um eine Zeile nach oben kopieren
  // (ab Zeile Nr. 1)
  for(y=1;y<BasicEditorFont.char_anzy;y++) {
    for(x=0;x<BasicEditorFont.char_anzx;x++) {
      EDITOR_TEXT_RAM[x][y-1]=EDITOR_TEXT_RAM[x][y];
      EDITOR_COLOR_RAM[x][y-1]=EDITOR_COLOR_RAM[x][y];
    }
  }

  // letzte Zeile loeschen
  y=(BasicEditorFont.char_anzy-1);
  for(x=0;x<BasicEditorFont.char_anzx;x++) {
    EDITOR_TEXT_RAM[x][y]=EDITOR_SPACE_CHAR;
    EDITOR_COLOR_RAM[x][y]=p_Editor_getAktColor();
  }

  // Screen komplett kopieren
  p_Editor_updateScreenRAM(99);
}


//--------------------------------------------------------------
// interne Funktion
// verschiebt die aktuele Zeile an CursorPos
// um ein Zeichen nach links
// 1. das erste Zeichen geht damit verloren
// 2. das letzte Zeichen wird mit Space beschrieben
//--------------------------------------------------------------
void p_Editor_moveRowLeft(uint8_t offset)
{
  uint32_t x,y,start;

  start=BASIC_EDITOR.cursor.xpos+offset;

  if(start==0) return;

  // Text- und Color-RAM gleichzeitig um ein Zeichen nach links kopieren
  // (ab Cursorpos)
  y=BASIC_EDITOR.cursor.ypos;

  for(x=start;x<BasicEditorFont.char_anzx;x++) {
    EDITOR_TEXT_RAM[x-1][y]=EDITOR_TEXT_RAM[x][y];
    EDITOR_COLOR_RAM[x-1][y]=EDITOR_COLOR_RAM[x][y];
  }

  // letztes Zeichen loeschen
  x=(BasicEditorFont.char_anzx-1);
  EDITOR_TEXT_RAM[x][y]=EDITOR_SPACE_CHAR;
  EDITOR_COLOR_RAM[x][y]=p_Editor_getAktColor();

  // Zeile kopieren
  p_Editor_updateScreenRAM(y);
}


//--------------------------------------------------------------
// interne Funktion
// verschiebt die aktuele Zeile an CursorPos
// um ein Zeichen nach rechts
// 1. das letzte Zeichen geht damit verloren
// 2. das erste Zeichen wird mit Space beschrieben
//--------------------------------------------------------------
void p_Editor_moveRowRight(void)
{
  uint32_t x,y,ende;

  if(BASIC_EDITOR.cursor.xpos>=(BasicEditorFont.char_anzx-1)) return;

  // Text- und Color-RAM gleichzeitig um ein Zeichen nach rechts kopieren
  // (ab Cursorpos)
  y=BASIC_EDITOR.cursor.ypos;
  ende=BASIC_EDITOR.cursor.xpos;

  for(x=BasicEditorFont.char_anzx-1;x>ende;x--) {
    EDITOR_TEXT_RAM[x][y]=EDITOR_TEXT_RAM[x-1][y];
    EDITOR_COLOR_RAM[x][y]=EDITOR_COLOR_RAM[x-1][y];
  }

  // erstes Zeichen loeschen
  x=BASIC_EDITOR.cursor.xpos;
  EDITOR_TEXT_RAM[x][y]=EDITOR_SPACE_CHAR;
  EDITOR_COLOR_RAM[x][y]=p_Editor_getAktColor();

  // Zeile kopieren
  p_Editor_updateScreenRAM(y);
}


//--------------------------------------------------------------
// interne Funktion
// zeigt den Cursor an
// (je nach mode entweder normal oder invertiert)
//--------------------------------------------------------------
void p_Editor_showCursor(void)
{
  uint32_t x,y;
  uint8_t ascii_value,col;
  uint16_t vg,bg;
  uint16_t xp,yp;

  // Cursor-Position ermitteln
  x=BASIC_EDITOR.cursor.xpos;
  y=BASIC_EDITOR.cursor.ypos;

  // Zeichen und Farbe ermitteln
  ascii_value=EDITOR_TEXT_RAM[x][y];
  col=EDITOR_COLOR_RAM[x][y];

  vg=EDITOR_COLOR_TABLE_VG[col&0x0F];
  bg=EDITOR_COLOR_TABLE_BG[col>>4];

  // Pixel-Position auf dem Display ermitteln
  xp=x*BasicEditorFont.font_name->width;
  yp=y*BasicEditorFont.font_name->height;

  // Cursor darstellen
  if(BASIC_EDITOR.cursor.visible==0) {
    // zeichen normal zeichnen
    UB_Font_BasicChar(xp,yp,ascii_value,BasicEditorFont.font_name,vg,bg);
  }
  else {
    // zeichen invertiert zeichnen
    UB_Font_BasicChar(xp,yp,ascii_value,BasicEditorFont.font_name,bg,vg);
  }
}


//--------------------------------------------------------------
// interne Funktion
// kopiert das Text/Color-RAM in den LCD-Buffer
// entweder eine einzelne Zeile oder den kompletten Screen
// Durchlaufzeit fuer eine Zeile ca. 850us
// Durchlaufzeit fuer den kompletten Screen ca. 25ms
// ToDo : screen koennte man auch per DMA um eine Zeile verschieben
//--------------------------------------------------------------
void p_Editor_updateScreenRAM(uint8_t row)
{
  uint32_t x,y;
  uint8_t ascii_value,col;
  uint16_t vg,bg;
  uint16_t xp,yp;
 
  if(row>=BasicEditorFont.char_anzy) {
    // den kompletten Screen kopieren
    yp=0;
    for(y=0;y<BasicEditorFont.char_anzy;y++) {
      xp=0;
      for(x=0;x<BasicEditorFont.char_anzx;x++) {
        // Zeichen und Farbe ermitteln
        ascii_value=EDITOR_TEXT_RAM[x][y];
        col=EDITOR_COLOR_RAM[x][y]; 
        vg=EDITOR_COLOR_TABLE_VG[col&0x0F];
        bg=EDITOR_COLOR_TABLE_BG[col>>4];       
        // zeichen kopieren
        UB_Font_BasicChar(xp,yp,ascii_value,BasicEditorFont.font_name,vg,bg);
        xp+=BasicEditorFont.font_name->width;
      }
      yp+=BasicEditorFont.font_name->height;
    }   
  }
  else {
    // nur eine einzelne Zeile kopieren
    xp=0;
    yp=row*BasicEditorFont.font_name->height;
    y=row;
    for(x=0;x<BasicEditorFont.char_anzx;x++) {
      // Zeichen und Farbe ermitteln
      ascii_value=EDITOR_TEXT_RAM[x][y];
      col=EDITOR_COLOR_RAM[x][y]; 
      vg=EDITOR_COLOR_TABLE_VG[col&0x0F];
      bg=EDITOR_COLOR_TABLE_BG[col>>4];       
      // zeichen kopieren
      UB_Font_BasicChar(xp,yp,ascii_value,BasicEditorFont.font_name,vg,bg);
      xp+=BasicEditorFont.font_name->width;
    }    
  }
}
