//--------------------------------------------------------------
// File     : stm32_ub_basic_editor.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_BASIC_EDITOR_H
#define __STM32F4_UB_BASIC_EDITOR_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_graphic2d.h"
#include "stm32_ub_font.h"



//--------------------------------------------------------------
// Standardfarben vom Editor (16 Farben)
// 16bit Farbwert (R5G6B5)
// Red   (5bit) -> Bit15-Bit11
// Green (6bit) -> Bit10-Bit5
// Blue  (5bit) -> Bit4-Bit0
//--------------------------------------------------------------
#define  EDITOR_COL_0  0x0000 // schwarz
#define  EDITOR_COL_1  0xFFFF // weiss
#define  EDITOR_COL_2  0xF800 // rot
#define  EDITOR_COL_3  0xAFFD // tuerkis
#define  EDITOR_COL_4  0xCA39 // violett
#define  EDITOR_COL_5  0x07E0 // gruen
#define  EDITOR_COL_6  0x421C // blau
#define  EDITOR_COL_7  0xFFE0 // gelb
#define  EDITOR_COL_8  0xDC4A // orange
#define  EDITOR_COL_9  0x6220 // braun
#define  EDITOR_COL_10 0xFBAE // hellrot
#define  EDITOR_COL_11 0x3186 // grau 1
#define  EDITOR_COL_12 0x73AE // grau 2
#define  EDITOR_COL_13 0xAFEC // hellgruen
#define  EDITOR_COL_14 0xA53F // hellblau
#define  EDITOR_COL_15 0xBDD7 // grau 3



//--------------------------------------------------------------
// Farbtabellen im RAM
//--------------------------------------------------------------
uint16_t  EDITOR_COLOR_TABLE_VG[16]; // Text-Farbe
uint16_t  EDITOR_COLOR_TABLE_BG[16]; // Hintergrund-Farbe



//--------------------------------------------------------------
// Groesse vom Editorbereich (bei 8x8 Pixel Font)
//--------------------------------------------------------------
#define  BASIC_EDITOR_8x8_ANZX   40  // anzahl der Zeichen (in X)
#define  BASIC_EDITOR_8x8_ANZY   30  // anzahl der Zeichen (in Y)



//--------------------------------------------------------------
// Farb default Settings
//--------------------------------------------------------------
#define  EDITOR_DEF_VG_COL    1      // textfarbe    1 = weiss
#define  EDITOR_DEF_BG_COL    6      // hintergrund  6 = blau
#define  EDITOR_DEF_SYS_COL   5      // system       5 = gruen



//--------------------------------------------------------------
// RAM vom Editor
// Text-Ram  = Ascii-Codes [0...255]
// Color-RAM = Farbdaten
//              [Bit 0...3 = Textfarbe]
//              [Bit 4...7 = Hintergrundfarbe]
//
// RAM[x][y] : x=[0...39] : 0=links
//             y=[0...29] : 0=oben
//--------------------------------------------------------------
uint8_t  EDITOR_TEXT_RAM[BASIC_EDITOR_8x8_ANZX][BASIC_EDITOR_8x8_ANZY];
uint8_t  EDITOR_COLOR_RAM[BASIC_EDITOR_8x8_ANZX][BASIC_EDITOR_8x8_ANZY];



//--------------------------------------------------------------
// sonstige defines
//--------------------------------------------------------------
#define  EDITOR_FIRST_CHAR          32  // erstes Zeichen vom Font
#define  EDITOR_LAST_CHAR          126  // letztes Zeichen vom Font
#define  EDITOR_SPACE_CHAR          32  // ascii=space
#define  EDITOR_LINEFEED            10  // Linefeed-Teichen "\n"


//--------------------------------------------------------------
// enum fuer Cursorbewegung
//--------------------------------------------------------------
typedef enum {
  MOVE_CUR_UP =0,     // hoch (y--)
  MOVE_CUR_DOWN,      // runter (y++)
  MOVE_CUR_LEFT,      // links (x--)
  MOVE_CUR_RIGHT,     // rechts (x++)
  MOVE_CUR_X0,        // linkes ende (x=0)
  MOVE_CUR_X0Y0,      // home position (0,0)
  MOVE_CUR_NEXTLINE,  // naechste zeile (x=0, y++)
  MOVE_CUR_LASTCHAR   // letztes zeichen in zeile
}MOVE_CURSOR_t;



//--------------------------------------------------------------
// aktuell eingestellter Zeichensatz
//--------------------------------------------------------------
typedef struct {
  uint8_t font_nr;    // Font-Nummer [0...3]
  UB_Font *font_name; // Pointer auf die Font-Struktur
  uint32_t char_anzx; // max Anzahl Zeichen in X-Richtung
  uint32_t char_anzy; // max Anzahl Zeichen in Y-Richtung
}BASIC_FONT_t;
BASIC_FONT_t BasicEditorFont;      // aktueller Font


//--------------------------------------------------------------
// aktuell eingestellte Farbe
//--------------------------------------------------------------
typedef struct {
  uint8_t vg_col;     // Text-Farbe         [0...15]  
  uint8_t bg_col;     // Hintergrund-Farbe  [0...15]
  uint8_t sys_col;    // System-Farbe       [0...15]
}BASIC_COLOR_t;



//--------------------------------------------------------------
// aktuelle Position vom Cursor
//--------------------------------------------------------------
typedef struct {
  uint32_t xpos;      // (0...BASIC_EDITOR_ANZX-1) 0=links
  uint32_t ypos;      // (0...BASIC_EDITOR_ANZY-1) 0=oben
  uint8_t visible;    // 0=normal , 1=inverse
}BASIC_CURSOR_t;



//--------------------------------------------------------------
// Struktur vom Basic-Editor
//--------------------------------------------------------------
typedef struct {
  BASIC_COLOR_t color;    // aktuelle Farbe (Text/Hintergrund)
  BASIC_CURSOR_t cursor;  // aktuelle Cursor Position
}BASIC_EDITOR_t;




//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_Basic_Editor_Init(void);
void UB_Basic_Editor_Reset(void);
void UB_Basic_Editor_ClearSreen(void);
void UB_Basic_Editor_UpdateSreen(void);
void UB_Basic_Editor_WriteChar(uint8_t ascii_value);
void UB_Basic_Editor_WriteString(char *ptr);
void UB_Basic_Editor_WriteSystemString(char *ptr);
void UB_Basic_Editor_InsertChar(uint8_t ascii_value);
void UB_Basic_Editor_BackspaceChar(void);
void UB_Basic_Editor_DeleteChar(void);
void UB_Basic_Editor_MoveCursor(MOVE_CURSOR_t direction);
void UB_Basic_Editor_SetCursor(uint32_t x, uint32_t y);
void UB_Basic_Editor_ToggleCursor(void);
void UB_Basic_Editor_SetVGColor(uint8_t color);
uint16_t UB_Basic_Editor_GetVGColor(void);
void UB_Basic_Editor_SetBGColor(uint8_t color);
uint16_t UB_Basic_Editor_GetBGColor(void);
void UB_Basic_Editor_SetSYSColor(uint8_t color);
void UB_Basic_Editor_ChangeFont(void);
uint8_t UB_Basic_Editor_CopyLine(char *ptr);
void UB_Basic_Editor_DebugString(char *ptr);



//--------------------------------------------------------------
#endif // __STM32F4_UB_BASIC_EDITOR_H
