//--------------------------------------------------------------
// File     : stm32_ub_basic_interpreter.c
// Datum    : 20.12.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : /
// Funktion : Basic-Interpreter zusatzfunktionen
//--------------------------------------------------------------




//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_basic_interpreter.h"


extern char data_var_array_data[]; // UB
char com1_buf[85];

//--------------------------------------------------------------
void p_Interpreter_CheckPayload(char *ptr, uint8_t token, uint8_t cmd_len);
uint8_t isdigit_ub(uint8_t ascii_nr);
uint32_t p_Interpreter_get_number(char *ptr);
uint8_t p_Interpreter_SearchNextLine(char *programm, uint32_t start);
void p_Interpreter_DeleteZeile(char *programm);
void p_Interpreter_InsertZeile(char *programm, char *ptr);
uint8_t p_Interpreter_calcGPIO(uint32_t nr, uint32_t typ);
uint8_t p_Interpreter_calcADC(uint32_t nr);
void p_Interpreter_initADC(void);
uint16_t p_Interpreter_readADC(void);
void p_Interpreter_initSPI4(uint16_t v1, uint16_t v2);


//--------------------------------------------------------------
// Struktur der Token
//--------------------------------------------------------------
struct command_token {
  char *command;
  uint8_t token;
};



//--------------------------------------------------------------
// Liste aller Inline-Interpreter Befehle
// commands in UPPERCASE !!
// sorted by commandlen
//--------------------------------------------------------------
static const struct command_token commands[] = {
  // commands for editor-mode
  {"TERMINAL", COMMAND_TERMINAL},
  {"VERSION", COMMAND_VERSION},
  {"DELETE", COMMAND_DELETE},
  {"REMOVE", COMMAND_REMOVE},
  {"DEVICE", COMMAND_DEVICE},
  {"RESET", COMMAND_RESET},
  {"PARSE", COMMAND_PARSE},  
  {"LIST", COMMAND_LIST},
  {"LOAD", COMMAND_LOAD},
  {"SAVE", COMMAND_SAVE},
  {"FREE", COMMAND_FREE},
  {"NEW", COMMAND_NEW},
  {"CLS", COMMAND_CLS},
  {"RUN", COMMAND_RUN},
  {"DIR", COMMAND_DIR},
  {"CD", COMMAND_CD},
  //------------------------------------
  // inline basic-commands
  {"RIGHT$", COMMAND_BASIC_KEYWORD_AUTOESC},
  {"PRINT", COMMAND_BASIC_KEYWORD},
  {"LET", COMMAND_BASIC_KEYWORD_AUTOESC},
  {"DIM", COMMAND_BASIC_KEYWORD_AUTOESC},
  {"ABS", COMMAND_BASIC_KEYWORD_AUTOESC},
  {"IF", COMMAND_BASIC_KEYWORD},
  //------------------------------------
  {NULL, COMMAND_ERROR}
};




//--------------------------------------------------------------
// testet den uebergebenen String
// und sucht nach dem passenden inline Interpreter Kommando
//
// fuellt die Struktur :
//  INTERPRETER.command = gefundenes Token
//  INTERPRETER.merker  = Bedeutung je nach Token
//--------------------------------------------------------------
void UB_Basic_Interpreter_CheckString(char *ptr)
{
  uint8_t ok,i,cmd_len;
  struct command_token const *ct;
  uint32_t nr;

  // Struktur zuruecksetzen
  INTERPRETER.command=COMMAND_ERROR;
  INTERPRETER.merker=0;

  // alle moeglichen Token durchgehen
  for(ct = commands; ct->command != NULL; ++ct) {
    cmd_len=strlen(ct->command);
    if(strncmp(ptr, ct->command, cmd_len) == 0) {
      // token gefunden
      p_Interpreter_CheckPayload(ptr, ct->token, cmd_len);
      return;
    }
    else {
      // check upercase & lowercase
      ok=0;
      for(i=0;i<cmd_len;i++) {
        if((*(ptr+i)!=ct->command[i]) && (*(ptr+i)!=(ct->command[i]+0x20))) {
          // zeichen falsch
          ok=1;
          break;
        }
      }
      if(ok==0) {
        // token gefunden
        p_Interpreter_CheckPayload(ptr, ct->token, cmd_len);
        return;
      }
    }
  }

  // Token nicht bekannt
  // check auf Zahl bzw. Basic-Zeile
  if(isdigit_ub(ptr[0])) {
    // zahl gefunden
    nr=p_Interpreter_get_number(ptr);
    if(strlen(ptr) > INTERPRETER.nr_len) {
      // eine Programmzeile wurde eingegeben
      INTERPRETER.command=COMMAND_CODELINE;
      // zahl zurueckliefern
      INTERPRETER.merker=nr;
    }
  }
}


//--------------------------------------------------------------
// sucht die Adresse einer Basic-Zeile im RAM
// ret_wert : adresse der Zeile (start oder ende)
//            0=falls Zeilennummer nicht gefunden wurde
//--------------------------------------------------------------
uint32_t UB_Basic_Interpreter_GetLineAdr(char *programm, uint32_t linenum, uint8_t mode)
{
  uint32_t ret_wert=0;
  uint8_t check;
  uint32_t ram_adr=0;

  // check auf gueltige Zeilen-Nummern
  if(linenum==0) return(0);

  // suchen der Zeilennummer im kompletten Programm
  do {
    // check ob eine Zeilennumer gefunden wurde
    check=p_Interpreter_SearchNextLine(programm,ram_adr);
    if((check&0x01)!=0) {
      // auf naechste zeile schalten
      ram_adr=INTERPRETER.line_end;
      // check ob gesuchte zeilen-nummer
      if(INTERPRETER.line_nr>=linenum) {
        // adresse der Zeile merken
    	if(mode==0) ret_wert=INTERPRETER.line_start;
    	if(mode==1) ret_wert=INTERPRETER.line_end;
        check|=0x04; // schleife verlassen
      }
    }
  }
  while((check&0x06)==0); // bis EOF oder Zeile gefunden

  return(ret_wert);
}

//--------------------------------------------------------------
// loescht eine komplette Zeile vom Programm
// mit einer bestimmten Zeilen-Nummer
//
// program : Zeiger auf das Basic-Program
// linenum : Zeilen-Nummer die geloescht werden soll
//
// fuellt die Struktur :
//  INTERPRETER.line_nr   = Zeilen Nummer die geloescht wurde (0=keine)
//  INTERPRETER.prg_size  = laenge vom Programm
//--------------------------------------------------------------
void UB_Basic_Interpreter_DeleteOneLine(char *programm, uint32_t linenum)
{
  uint8_t check;
  uint32_t ram_adr=0;


  INTERPRETER.line_nr=0;
  // check auf gueltige Zeilen-Nummern
  if(linenum==0) return;

  // suchen der Zeilennummer im kompletten Programm
  do {
    // check ob eine Zeilennumer gefunden wurde
    check=p_Interpreter_SearchNextLine(programm,ram_adr);    
    if((check&0x01)!=0) {      
      // auf naechste zeile schalten
      ram_adr=INTERPRETER.line_end;
      // check ob gesuchte zeilen-nummer
      if(INTERPRETER.line_nr==linenum) {
        // zeile loeschen
        p_Interpreter_DeleteZeile(programm);
        check|=0x04; // schleife verlassen        
      }
    }
  }
  while((check&0x06)==0); // bis EOF oder Zeile gefunden

  if((check&0x04)==0) {
    // wenn Zeilen-Nummer nicht gefunden wurde
    INTERPRETER.line_nr=0;
  }

}


//--------------------------------------------------------------
// fuegt eine komplette Zeile mit einer bestimmten
// Zeilennummer in das Programm ein
// (falls die Zeilen-Nummer schon exisitert, wird diese zuvor geloescht)
//
// program : Zeiger auf das Basic-Program
// linenum : Zeilen-Nummer die eingefuegt werden soll
// ptr     : Zeiger auf die Zeile die eingefuegt werden soll
//
// fuellt die Struktur :
//  INTERPRETER.prg_size  = laenge vom Programm nach dem einfuegen
//--------------------------------------------------------------
void UB_Basic_Interpreter_InsertOneLine(char *programm, uint32_t linenum, char *ptr)
{
  uint8_t check;
  uint32_t ram_adr=0;
  uint32_t akt_linenr, prev_linenr=0, next_linenr=0;
  uint32_t ziel_adr=0;

  // check auf gueltige Zeilen-Nummern
  if(linenum==0) return;

  // loescht die Zeile (falls es sie schon gibt)
  UB_Basic_Interpreter_DeleteOneLine(programm, linenum);

  // suchen der Stelle zum einfuegen der Zeile
  do {
    // check ob eine Zeilennumer gefunden wurde
    check=p_Interpreter_SearchNextLine(programm,ram_adr);
    if((check&0x01)!=0) {       
      // auf naechste zeile schalten
      ram_adr=INTERPRETER.line_end;
      // zeilen nummern mit dem ziel vergleichen
      akt_linenr=INTERPRETER.line_nr;
      if(akt_linenr<=linenum) prev_linenr=akt_linenr;
      if(akt_linenr>=linenum) next_linenr=akt_linenr;

      // check ob stelle zum einfeugen gefunden
      if((next_linenr>0) && (prev_linenr>0)) {
        // schleife verlassen
        check|=0x04;
        // adresse speichern
        ziel_adr=INTERPRETER.line_start;
      }
    }
  }
  while((check&0x06)==0); // bis EOF oder position gefunden


  // Zeile an gefundener Stelle einfuegen
  if((check&0x04)!=0) {
    // wenn insert mitten im programm
    INTERPRETER.line_start=ziel_adr;    
    p_Interpreter_InsertZeile(programm, ptr);
  }
  else {
    // check wo zeile eingefuegt werden soll
    if((prev_linenr==0) && (next_linenr!=0)) {
      // Zeile ganz am anfang einfuegen
      INTERPRETER.line_start=0;      
      p_Interpreter_InsertZeile(programm, ptr);
    }
    else if((prev_linenr!=0) && (next_linenr==0)) {
      // Zeile ganz am ende einfuegen
      INTERPRETER.line_start=INTERPRETER.prg_size-1;      
      p_Interpreter_InsertZeile(programm, ptr);
    }
    else if((prev_linenr==0) && (next_linenr==0)) {
      // Zeile im leeren Programm einfuegen
      INTERPRETER.line_start=0;      
      p_Interpreter_InsertZeile(programm, ptr);
    }
  }
}



//--------------------------------------------------------------
// kopiert empfangenen String von COM-1
//--------------------------------------------------------------
void UB_Basic_Interpreter_COPY_RXString(char *ptr)
{
  uint32_t n;
  uint8_t wert;

  for(n=0;n<85;n++) {
    wert=ptr[n];
    com1_buf[n]=wert;
    if(wert==0x00) break;
  }
  INTERPRETER.com1_merker=1;
}


//--------------------------------------------------------------
// interne Funktion
// sucht in einem Teil von einem String eine Zahl
// oder ein String oder ein Minuszeichen
//
// mode : 0=suche zahl   -> INTERPRETER.nr_val
//        1=suche string -> INTERPRETER.filename
//        2=suche minus  -> /
// ret_wert : 0=error, 1=ok
//--------------------------------------------------------------
uint8_t checkString(uint8_t mode, char *ptr, uint8_t start, uint8_t *stop)
{
  uint8_t ret_wert=0;
  uint8_t str_len,pos,wert;
  uint32_t n;

  str_len=strlen(ptr);
  *stop=str_len;
  pos=start;
  if(pos>=str_len) return 0;

  switch(mode) {
    case 0 : // check auf zahl
      do {
        if(ptr[pos]==' ') pos++;
        else if(isdigit_ub(ptr[pos])==1) {
          n=p_Interpreter_get_number(ptr+pos);
          INTERPRETER.nr_val=n;
          *stop=pos+INTERPRETER.nr_len;
          // zahl gefunden
          return(1);
        }
        else break;
      }while(pos<str_len);
    break;
    case 1 : // check auf string
      do {
        if(ptr[pos]==' ') pos++;
        else if(ptr[pos]=='"') {
          pos++;
          for(n=0;n<14;n++) {
            if(pos>=str_len) break;
            wert=ptr[pos];
            pos++;
            if(wert=='"') break;
            INTERPRETER.filename[n]=wert;
          }
          INTERPRETER.filename[n]=0x00;
          *stop=pos;
          // string gefunden
          return(1);
        }
        else break;
      }while(pos<str_len);
    break;
    case 2 : // check auf minus
      do {
        if(ptr[pos]==' ') pos++;
        else if(ptr[pos]=='-') {
          pos++;
          *stop=pos;
          // minus gefunden
          return(1);
        }
        else break;
      }while(pos<str_len);
    break;
  }

  *stop=pos;

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// prueft String von einem gefundenen Token nach Parametern
// je nach Token
//
// ptr     = eingegebenen String
// token   = Kommando
// cmd_len = laenge vom Token
//
// fuellt die Struktur :
//  INTERPRETER.command  = gefundenes Token
//  INTERPRETER.merker   = Bedeutung je nach Token
//  INTERPRETER.filename = bei LOAD/SAVE/REMOVE
//--------------------------------------------------------------
void p_Interpreter_CheckPayload(char *ptr, uint8_t token, uint8_t cmd_len)
{
  uint8_t n,check;
  uint8_t stop;

  INTERPRETER.command=token;

  switch(token) {
    case COMMAND_DEVICE :
    case COMMAND_TERMINAL :
      // token = DEVICE oder TERMINAL
      // Syntax => "DEVICE n" n=Device
      INTERPRETER.command=token;
      INTERPRETER.merker=0;
      // zahl auslesen
      check=checkString(0,ptr,cmd_len,&stop);
      if(check==0) {
        INTERPRETER.command=COMMAND_ERROR;
        return;
      }
      INTERPRETER.merker=INTERPRETER.nr_val;
    break;
    case COMMAND_LOAD :
    case COMMAND_SAVE :
    case COMMAND_REMOVE :
    case COMMAND_CD :
      // token = LOAD oder SAVE oder REMOVE
      // Syntax => LOAD ""
      // Syntax => SAVE "Test.bas"
      INTERPRETER.command=token;
      INTERPRETER.merker=0;      
      // string auslesen
      check=checkString(1,ptr,cmd_len,&stop);
      if(check==0) {
        INTERPRETER.command=COMMAND_ERROR;
        return;
      }
    break;
    case COMMAND_DELETE :
      // token = DELETE
      // Syntax = "DELETE n" n=Zeilen-Nummer
      INTERPRETER.command=COMMAND_DELETE;
      INTERPRETER.merker=0;
      // zahl auslesen
      check=checkString(0,ptr,cmd_len,&stop);
      if(check==0) {
        INTERPRETER.command=COMMAND_ERROR;
        return;
      }
      INTERPRETER.merker=INTERPRETER.nr_val;
    break;
    case COMMAND_LIST :
      // token LIST
      // Syntax "List n-m" n,m=Zeilen-Nummer
      INTERPRETER.command=COMMAND_LIST;
      INTERPRETER.merker=0;
      INTERPRETER.z1=0;
      INTERPRETER.z2=0;
      // check ob nur 'list'
      n=strlen(ptr);
      if(n==cmd_len) return;
      // 1. zahl auslesen
      check=checkString(0,ptr,cmd_len,&stop);
      if(check==1) {
    	  INTERPRETER.merker|=0x01;
    	  INTERPRETER.z1=INTERPRETER.nr_val;
      }
      n=stop;
      // minus auslesen
      check=checkString(2,ptr,n,&stop);
      if(check==1) INTERPRETER.merker|=0x02;
      n=stop;
      // 2. zahl auslesen
      check=checkString(0,ptr,n,&stop);
      if(check==1) {
    	  INTERPRETER.merker|=0x04;
    	  INTERPRETER.z2=INTERPRETER.nr_val;
      }
      if(INTERPRETER.merker==0) {
        INTERPRETER.command=COMMAND_ERROR;
        return;
      }
    break;
    case COMMAND_BASIC_KEYWORD :
    case COMMAND_BASIC_KEYWORD_AUTOESC :
      INTERPRETER.command=token;
      INTERPRETER.merker=0;
    break;
    default :
      // normales Token ohne Parameter
      // check ob laenge stimmt
      n=strlen(ptr);
      if(n!=cmd_len) INTERPRETER.command=COMMAND_ERROR;
  }
}


//--------------------------------------------------------------
// interne Funktion
// check ob ascii-zeichen eine ziffer ist (0...9)
// ret_wert : 0=keine Ziffer, 1=Ziffer
//--------------------------------------------------------------
uint8_t isdigit_ub(uint8_t ascii_nr)
{
  if(ascii_nr<48) return(0);
  if(ascii_nr>57) return(0);
  return(1);
}


//--------------------------------------------------------------
// interne Funktion
// wandelt einen String in eine Zahl im
// ret_wert = Zahl [0...999999]
//
// fuellt die Struktur :
//  INTERPRETER.nr_len = anzahl der ziffern
//--------------------------------------------------------------
uint32_t p_Interpreter_get_number(char *ptr)
{
  uint32_t ret_wert=0,n;

  INTERPRETER.nr_len=0;
  for(n=0;n<MAX_NUMLEN;n++) {
    if(!isdigit_ub(ptr[n])) {
      if(n > 0) {
        ret_wert=atoi(ptr);
        INTERPRETER.nr_len=n;
        return(ret_wert);
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// sucht naechste Zeilen-Nummer (ab start_adr)
// ret_wert : bit0 : 1, wenn eine Zeilennummer gefunden wurde
//            bit1 : 1, wenn Ende vom Programm erreicht
//
// fuellt die Struktur :
// INTERPRETER.line_nr    = gefundene Zeilen Nummer (0 falls keine)
// INTERPRETER.line_start = start adresse der Zeile im RAM
// INTERPRETER.line_end   = end adresse der Zeile im RAM
//--------------------------------------------------------------
uint8_t p_Interpreter_SearchNextLine(char *programm, uint32_t start)
{
  uint8_t ret_wert=0;
  uint32_t ram_adr=0;
  uint8_t wert;
  char *ptr;
  uint8_t linefound_flag=0;
  uint8_t eof_flag=0,cr_flag=0;

  // variabeln loeschen
  INTERPRETER.line_nr=0;
  INTERPRETER.line_start=0;
  INTERPRETER.line_end=0;
  ram_adr=start;

  // nachste Zeile mit Zeilen-Nummer suchen
  do {
    wert=programm[ram_adr];
    if(isdigit_ub(wert)) {
      // Zahl am Anfang einer Zeile gefunden = Zeilen-Nummer
      ptr=programm+ram_adr;
      // Zeilen-Nummer auslesen
      INTERPRETER.line_nr=p_Interpreter_get_number(ptr);
      linefound_flag=1;
      // start adresse merken
      INTERPRETER.line_start=ram_adr;
    }
    // skipline
    cr_flag=0;
    do {
      wert=programm[ram_adr];
      if(wert==0) eof_flag=1; // ende vom File
      if(wert==10) cr_flag=1; // ende einer Zeile
      ram_adr++;
    }while((eof_flag==0) && (cr_flag==0));
  }while((eof_flag==0) && (linefound_flag==0));

  // check ob eine Zeilen-Nummer gefunden wurde
  if(linefound_flag==1) {
    ret_wert|=0x01;
    // end adresse merken
    INTERPRETER.line_end=ram_adr;    
  }

  // check ob ende vom File
  if(eof_flag==1) {
    ret_wert|=0x02; 
    INTERPRETER.prg_size=ram_adr;    
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// loescht eine Zeile im RAM
// start = INTERPRETER.line_start
// ende  = INTERPRETER.line_end
//
// fuellt die Struktur :
// INTERPRETER.prg_size = grosse vom Programm nach dem loeschen
//--------------------------------------------------------------
void p_Interpreter_DeleteZeile(char *programm)
{
  uint8_t wert;
  uint32_t ziel_adr,quell_adr;
  uint8_t ende_flag=0;

  // quell und ziel adresse festlegen
  ziel_adr=INTERPRETER.line_start;
  quell_adr=INTERPRETER.line_end;

  // komplette Zeile loeschen
  do {
    wert=programm[quell_adr];
    programm[ziel_adr]=wert;
    if(wert==0) ende_flag=1;
    quell_adr++;
    ziel_adr++;
  }while(ende_flag==0);

  // Neue Programm laenge zurueckgeben
  INTERPRETER.prg_size=ziel_adr;
}


//--------------------------------------------------------------
// interne Funktion
// fuegt eine Zeile ins RAM ein
// start = INTERPRETER.line_start
// ptr   = Zeile die eingefuegt wird
//
// fuellt die Struktur :
// INTERPRETER.prg_size = grosse vom Programm nach dem einfuegen
//--------------------------------------------------------------
void p_Interpreter_InsertZeile(char *programm, char *ptr)
{
  uint32_t laenge,anzahl,n;
  uint8_t wert;
  uint32_t ziel_adr,quell_adr;

  // laenge und anzahl ermitteln
  laenge=strlen(ptr);
  anzahl=INTERPRETER.prg_size-INTERPRETER.line_start;

  // quell und ziel adresse festlegen
  quell_adr=INTERPRETER.prg_size-1;
  ziel_adr=INTERPRETER.prg_size+laenge;

  // kompletten Block verschieben
  // um platz zu schaffen
  for(n=0;n<anzahl;n++) {
    wert=programm[quell_adr];
    programm[ziel_adr]=wert;
    quell_adr--;
    ziel_adr--;
  }

  // Zeile einfuegen
  ziel_adr=INTERPRETER.line_start;
  for(n=0;n<laenge;n++) {
    wert=ptr[n];
    programm[ziel_adr+n]=wert;
  }
  // Zeilen-Endekennug hinzufuegen
  programm[ziel_adr+n]=0x0A;

  // Neue Programm laenge zurueckgeben
  INTERPRETER.prg_size+=laenge+1;  
}











