//--------------------------------------------------------------
// File     : stm32_ub_basic_interpreter.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_BASIC_INTERPRETER_H
#define __STM32F4_UB_BASIC_INTERPRETER_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include <string.h>
#include <stdlib.h>
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_adc.h"
#include "stm32_ub_uart.h"
#include "stm32_ub_i2c3.h"
#include "stm32_ub_spi5.h" 
#include "stm32_ub_spi4.h"


#define   MAX_NUMLEN  6



//--------------------------------------------------------------
// ADC-Clock
// Max-ADC-Frq = 36MHz
// Grundfrequenz = APB2 (APB2=90MHz)
// Moegliche Vorteiler = 2,4,6,8
//--------------------------------------------------------------

//#define ADC1s_VORTEILER     ADC_Prescaler_Div2 // Frq = 45 MHz
#define ADC1s_VORTEILER     ADC_Prescaler_Div4   // Frq = 22.5 MHz
//#define ADC1s_VORTEILER     ADC_Prescaler_Div6 // Frq = 15 MHz
//#define ADC1s_VORTEILER     ADC_Prescaler_Div8 // Frq = 11.25 MHz 



//--------------------------------------------------------------
// enums der inline Token
//--------------------------------------------------------------
enum {
  COMMAND_ERROR =0,
  COMMAND_RESET,
  COMMAND_FREE,
  COMMAND_VERSION,
  COMMAND_NEW,
  COMMAND_CLS,
  COMMAND_LIST,
  COMMAND_LOAD,
  COMMAND_SAVE,
  COMMAND_PARSE,
  COMMAND_RUN,
  COMMAND_TERMINAL,
  COMMAND_DIR,
  COMMAND_REMOVE,
  COMMAND_DELETE,
  COMMAND_DEVICE,
  COMMAND_CD,
  COMMAND_CODELINE,
  COMMAND_BASIC_KEYWORD,
  COMMAND_BASIC_KEYWORD_AUTOESC,
};


//--------------------------------------------------------------
// Struktur vom Interpreter
//--------------------------------------------------------------
typedef struct {
  uint8_t command;      // Command-Token
  uint32_t merker;      // Merker
  uint32_t z1;          // Merker fuer Zahl1
  uint32_t z2;          // Merker fuer Zahl2
  char filename[15];    // filename
  uint32_t nr_val;      // wert einer Zahl
  uint32_t nr_len;      // laenge einer Zahl
  uint32_t prg_size;    // laenge vom Programm
  uint32_t line_nr;     // Zeilen-Nummer
  uint32_t line_start;  // Start der Zeile im RAM
  uint32_t line_end;    // Ende der Zeile im RAM
  GPIO_TypeDef* gpio_port; // GPIO Port
  uint16_t gpio_pin;       // GPIO Pin
  uint8_t adc_ch;          // ADC Kanal
  uint8_t key_nr1;      // Nr. der gedrueckten Taste-1
  uint8_t key_nr2;      // Nr. der gedrueckten Taste-2
  uint8_t key_shift;    // Nr. der gedrueckten Shift-Tasten
  uint8_t chr_nr1;      // Ascii der gedrueckten Taste-1
  uint8_t chr_nr2;      // Ascii der gedrueckten Taste-2
  uint8_t touch_status; // Status vom Touch
  uint16_t touch_xp;    // Xpos vom Touch
  uint16_t touch_yp;    // ypos vom Touch
  uint8_t com1_merker;  // merker fuer com1 rx
  uint8_t init_port[5]; // merker fuer PORT init
  uint8_t init_adc;     // merker fuer ADC init
}INTERPRETER_t;
INTERPRETER_t INTERPRETER;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_Basic_Interpreter_CheckString(char *ptr);
uint32_t UB_Basic_Interpreter_GetLineAdr(char *programm, uint32_t linenum, uint8_t mode);
void UB_Basic_Interpreter_DeleteOneLine(char *programm, uint32_t linenum);
void UB_Basic_Interpreter_InsertOneLine(char *programm, uint32_t linenum, char *ptr);
void UB_Basic_Interpreter_GPIO_Init(uint32_t nr, uint32_t typ, uint32_t mode);
void UB_Basic_Interpreter_GPIO_Out(uint32_t nr, uint32_t value);
uint32_t UB_Basic_Interpreter_GPIO_In(uint32_t nr);
uint32_t UB_Basic_Interpreter_ADC_In(uint32_t nr, uint32_t mode);
uint32_t UB_Basic_Interpreter_TOUCH_Field(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
uint32_t UB_Basic_Interpreter_SQRT(uint32_t wert);
void UB_Basic_Interpreter_INIT_Port(uint32_t port, uint16_t v1, uint16_t v2);
void UB_Basic_Interpreter_SEND_Data(uint32_t port, uint32_t adr, uint32_t anzahl);
uint32_t UB_Basic_Interpreter_RECEIVE_Data(uint32_t port, uint32_t adr1, uint32_t anzahl, uint32_t adr2);
void UB_Basic_Interpreter_COPY_RXString(char *ptr);
 


//--------------------------------------------------------------
#endif // __STM32F4_UB_BASIC_INTERPRETER_H
