/*
 * Copyright (c) 2006, Adam Dunkels
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
#ifndef __UBASIC_H__
#define __UBASIC_H__

//#define  PLATFORM   1  //  1=PC
#define  PLATFORM   2  //  2=STM32F4


#if PLATFORM==1
#include "..\stm32f4xx.h"
#include "..\stm32_ub_dummy.h"
#endif


#if PLATFORM==2
#include "stm32_ub_graphic2d.h"
#include "stm32_ub_font.h"
#endif



void ubasic_init(char *program, int mode);
void ubasic_run(void);
int ubasic_finished(void);


//------------------------------------------------------------
int ubasic_get_int_var(int varnum);
void ubasic_set_int_var(int varnum, int value);
void ubasic_set_str_var(int varnum, char *ptr);                   // UB
char *ubasic_get_str_var(int varnum);                             // UB
void ubasic_set_int_var_array(int varnum, int varpos, int value); // UB
int ubasic_get_int_var_array(int varnum, int varpos);             // UB
void ubasic_set_str_var_array(int varnum, int varpos, char *ptr); // UB
char *ubasic_get_str_var_array(int varnum, int varpos);           // UB
void ubasic_set_data_var_array(int varnum, int varpos, int value);   // UB
int ubasic_get_data_var_array(int varnum, int varpos);               // UB
int ubasic_check_data_var_array(int varnum, int varpos, int length); // UB
void ubasic_set_datas_var_array(int varnum, int varpos, char *ptr);  // UB
char *ubasic_get_datas_var_array(int varnum, int varpos);            // UB
void ubasic_dim_int_array(int varnum, int length);                // UB
void ubasic_dim_str_array(int varnum, int length);                // UB
void ubasic_dim_data_array(int varnum, int length);               // UB
//------------------------------------------------------------



//------------------------------------------------------------
// syntax hints
//------------------------------------------------------------
//  [numbers]
//
//  <int_var>    : 'a' to 'z'
//  <int_array>  : 'A(n)' to 'Z(n)'
//  <data_array> : 'A#(n)' to 'Z#(n)'
//  <uint>       : '0' to '99999'
//  <int>        : '-99999' to '99999'
//  <int_cmd>    : e.g. ABS(n)
//
//  <int_expr>   : <int_var>,<int_array>,<data_array>,<int>,<int_cmd>
//
//------------------------------------------------------------
//  [strings]
//
//  <str_var>    : 'a$' to 'z$'
//  <str_array>  : 'A$(n)' to 'Z$(n)'
//  <str>        : e.g. "Test"
//  <str_cmd>    : e.g. MID$()
//
//  <str_expr>   : <str_var>,<str_array>,<str>,<str_cmd>
//
//------------------------------------------------------------
//  [other]
//
//  <cmd>        : e.g. PRINT
//  <void>       : without parameter
//  <printable>  : <int_expr>,<str_expr>
//  <tr>         : ';' or ','
//  <op>         : '+','-','*','/','%'
//  <rel>        : '=','>','<'
//
//------------------------------------------------------------







//------------------------------------------------------------
// TAB POSITIONS
//------------------------------------------------------------
#define  TAB_POS  80          // 80 Pixel per TAB



//------------------------------------------------------------
// Error-Numbers (by UB)
//------------------------------------------------------------
#define  UBASIC_PREPARSE_ERR       1
#define  UBASIC_UNEXPECTED_TOKEN   2
#define  UBASIC_INVALID_KEYWORD    3
#define  UBASIC_INVALID_DATA       4
#define  UBASIC_OVERFLOW           5
#define  UBASIC_OUT_OF_MEMORY      6



//------------------------------------------------------------
// global structure
//------------------------------------------------------------
typedef struct {
  uint16_t vg;        // text color
  uint16_t bg;        // background color
  uint32_t max_ram;   // max ram adr
  UB_Font *font_ptr;  // font
}UBASIC_t;
UBASIC_t UBASIC;



//------------------------------------------------------------
// NEW FUNCTIONS BY UB
//------------------------------------------------------------
int ubasic_failed(void);
int ubasic_linenum(void);
int ubasic_getkeyword(char token, char *ptr);
int ubasic_getramstatus(char mode);

#endif /* __UBASIC_H__ */
